from insights.parsers.proc_keys import ProcKeys
from insights.parsers import proc_keys, SkipException
from insights.tests import context_wrap
import doctest
import pytest

PROC_KEYS = """
009a2028 I--Q---   1 perm 3f010000  1000  1000 user     krb_ccache:primary: 12
1806c4ba I--Q---   1 perm 3f010000  1000  1000 keyring  _pid: 2
25d3a08f I--Q---   1 perm 1f3f0000  1000 65534 keyring  _uid_ses.1000: 1
28576bd8 I--Q---   3 perm 3f010000  1000  1000 keyring  _krb: 1
2c546d21 I--Q--- 190 perm 3f030000  1000  1000 keyring  _ses: 2
30a4e0be I------   4   2d 1f030000  1000 65534 keyring  _persistent.1000: 1
32100fab I--Q---   4 perm 1f3f0000  1000 65534 keyring  _uid.1000: 2
32a387ea I--Q---   1 perm 3f010000  1000  1000 keyring  _pid: 2
3ce56aea I--Q---   5 perm 3f030000  1000  1000 keyring  _ses: 1
""".strip()

PROC_KEYS_INVALID = """
unknow_case
""".strip()

PROC_KEYS_EMPTY = """
""".strip()


def test_etc_systemd():
    proc_keys_content = ProcKeys(context_wrap(PROC_KEYS))
    assert proc_keys_content[1]['id'] == '1806c4ba'
    assert proc_keys_content[1]['flags'] == 'I--Q---'
    assert proc_keys_content[1]['usage'] == '1'
    assert proc_keys_content[1]['timeout'] == 'perm'
    assert proc_keys_content[2]['permissions'] == '1f3f0000'
    assert proc_keys_content[2]['uid'] == '1000'
    assert proc_keys_content[2]['gid'] == '65534'
    assert proc_keys_content[2]['type'] == 'keyring'
    assert proc_keys_content[2]['description'] == '_uid_ses.1000: 1'

    assert proc_keys_content.search(timeout='perm')[0] == proc_keys_content[0]
    assert proc_keys_content.search(description__contains='pid')[0] == proc_keys_content[1]


def test_empty():
    with pytest.raises(SkipException) as e:
        ProcKeys(context_wrap(PROC_KEYS_EMPTY))
    assert 'No Contents' in str(e)

    with pytest.raises(SkipException) as e:
        ProcKeys(context_wrap(PROC_KEYS_INVALID))
    assert "Invalid Contents: unknow_case" in str(e)


def test_systemd_examples():
    env = {
        'proc_keys': ProcKeys(context_wrap(PROC_KEYS))
    }
    failed, total = doctest.testmod(proc_keys, globs=env)
    assert failed == 0



"""
virt-who configuration
======================
virt-who can accept configuration from several sources listed below in order
of precedence.

Configuration sources of virt-who::

    1. command line             # Ignored
    2. environment variables    # Ignored
    3. /etc/sysconfig/virt-who  # VirtWhoSysconfig
    4. /etc/virt-who.d/*.conf   # VirtWhoConf
    5. /etc/virt-who.conf       # VirtWhoConf

"""

from insights.core.plugins import combiner
from insights.parsers.sysconfig import VirtWhoSysconfig
from insights.parsers.virt_who_conf import VirtWhoConf


@combiner(VirtWhoSysconfig, VirtWhoConf)
class AllVirtWhoConf(object):
    """
    Combiner for accessing part of valid ``virt-who`` configurations

    Sample content of ``/etc/sysconfig/virt-who``::

        # Start virt-who on background, perform doublefork and monitor for virtual guest
        # events (if possible). It is NOT recommended to turn off this option for
        # starting virt-who as service.
        VIRTWHO_BACKGROUND=1
        # Enable debugging output.
        VIRTWHO_DEBUG=1
        # Send the list of guest IDs and exit immediately.
        VIRTWHO_ONE_SHOT=0
        # Acquire and send list of virtual guest each N seconds, 0 means default
        # configuration.
        VIRTWHO_INTERVAL=3600
        # Virt-who subscription manager backend. Enable only one option from the following:
        # Report to Subscription Asset Manager (SAM) or the Red Hat Customer Portal
        # Report to Sattellite version 6
        VIRTWHO_SATELLITE6=1


    Sample content of ``/etc/virt-who.conf``::

        [global]
        debug=False
        oneshot=True
        [defaults]
        owner=test

    Sample content of ``/etc/virt-who.d/satellite_esx.conf``::

        [esx_satellite]
        type=esx
        server=10.0.0.1
        owner=Satellite
        env=Satellite

    Examples:
        >>> shared = {VirtWhoSysconfig: vw_sys, VirtWhoConf: [vw_conf, vwd_conf]}
        >>> all_vw_conf = AllVirtWhoConf(None, shared)
        >>> all_vw_conf.background
        True
        >>> all_vw_conf.oneshot
        False  # `/etc/sysconfig/virt-who` has high priority
        >>> all_vw_conf.interval
        3600
        >>> all_vw_conf.sm_type
        'sat6'
        >>> all_vw_conf.hypervisors
        [{'name': 'esx_satellite', 'server': '10.0.0.1',
          'owner': 'Satellite', 'env': 'Satellite', type: 'esx'}]
        >>> all_vw_conf.hypervisor_types
        ['esx']


    Attributes:
        background(boolean): is ``virt-who`` running as a service
        oneshot(boolean): is ``virt-who`` running in ``one-shot`` mode or not
        interval(int): how often to check connected hypervisors for changes (seconds)
        sm_type(str): what subscription manager will ``virt-who`` report to
        hypervisors(list): list of dict of each connected hypervisors
        hypervisor_types(list): list of the connected hypervisors' type
    """

    def __init__(self, svw, vwc):
        self.background = False
        self.oneshot = True
        self.interval = 3600
        self.sm_type = 'cp'  # report to Red Hat Customer Portal by default
        self.hypervisors = []
        self.hypervisor_types = []

        # Check VirtWhoConf at first
        types = set()
        for c in vwc:
            if c.has_option('global', 'oneshot'):
                self.oneshot = c.getboolean('global', 'oneshot')
            if c.has_option('global', 'interval'):
                self.interval = c.getint('global', 'interval')
            for section in c.sections():
                # Check Hypervisor back-ends
                if c.has_option(section, 'type'):
                    hyper = {'name': section}
                    hyper.update(c.items(section))
                    types.add(hyper['type'])
                    self.hypervisors.append(hyper)
        self.hypervisor_types = list(types)

        self.background = ('1' == svw.get('VIRTWHO_BACKGROUND'))
        ost = svw.get('VIRTWHO_ONE_SHOT')
        self.oneshot = (ost == '1') if ost else self.oneshot
        itv = svw.get('VIRTWHO_INTERVAL')
        self.interval = int(itv) if itv and itv.isdigit() else self.interval

        # Check which subscription manager will report to
        if svw.get('VIRTWHO_SAM') == '1':
            self.sm_type = 'sam'  # report to SAM
        elif svw.get('VIRTWHO_SATELLITE6') == '1':
            self.sm_type = 'sat6'  # report to Satellite 6
        elif svw.get('VIRTWHO_SATELLITE5') == '1' or svw.get('VIRTWHO_SATELLITE') == '1':
            self.sm_type = 'sat5'  # report to Satellite 5



from insights.parsers.sysconfig import MemcachedSysconfig
from insights.tests import context_wrap

MEMCACHED_CONFIG = """
PORT="11211"
USER="memcached"
# max connection 2048
MAXCONN="2048"
# set ram size to 2048 - 2GiB
CACHESIZE="4096"
# disable UDP and listen to loopback ip 127.0.0.1, for network connection use real ip e.g., 10.0.0.5
OPTIONS="-U 0 -l 127.0.0.1"
""".strip()


def test_sysconfig_memcached():
    context = context_wrap(MEMCACHED_CONFIG, 'etc/sysconfig/memcached')
    conf = MemcachedSysconfig(context)

    assert 'OPTIONS' in conf
    assert 'FOO' not in conf
    assert conf.get('OPTIONS') == '-U 0 -l 127.0.0.1'
    assert conf.get('USER') == 'memcached'



"""
IsRhel6, IsRhel7, IsRhel8, and IsRhel9
======================================

An ``IsRhel*`` component is valid if the
:py:class:`insights.combiners.redhat_release.RedHatRelease` combiner indicates
the major RHEL version represented by the component. Otherwise, it raises a
:py:class:`insights.core.dr.SkipComponent` to prevent dependent components from
executing.

In particular, an ``IsRhel*`` component can be added as a dependency of a
parser to limit it to a given version.
"""
from insights.combiners.redhat_release import RedHatRelease
from insights.core.dr import SkipComponent
from insights.core.plugins import component


class IsRhel(object):
    """
    This component uses ``RedHatRelease`` combiner to determine the RHEL
    major version. It then checks if the major version matches the version
    argument, if it doesn't it raises ``SkipComponent``.

    Attributes:
        minor (int): The minor version of RHEL.

    Raises:
        SkipComponent: When RHEL major version does not match version.
    """
    def __init__(self, rhel, version=None):
        if rhel.major != version:
            raise SkipComponent("Not RHEL{vers}".format(vers=version))
        self.minor = rhel.minor


@component(RedHatRelease)
class IsRhel6(IsRhel):
    """
    This component uses ``RedHatRelease`` combiner
    to determine RHEL version. It checks if RHEL6, if not
    RHEL6 it raises ``SkipComponent``.

    Attributes:
        minor (int): The minor version of RHEL 6.

    Raises:
        SkipComponent: When RHEL version is not RHEL6.
    """
    def __init__(self, rhel):
        super(IsRhel6, self).__init__(rhel, 6)


@component(RedHatRelease)
class IsRhel7(IsRhel):
    """
    This component uses ``RedHatRelease`` combiner
    to determine RHEL version. It checks if RHEL7, if not
    RHEL7 it raises ``SkipComponent``.

    Attributes:
        minor (int): The minor version of RHEL 7.

    Raises:
        SkipComponent: When RHEL version is not RHEL7.
    """
    def __init__(self, rhel):
        super(IsRhel7, self).__init__(rhel, 7)


@component(RedHatRelease)
class IsRhel8(IsRhel):
    """
    This component uses ``RedhatRelease`` combiner
    to determine RHEL version. It checks if RHEL8, if not
    RHEL8 it raises ``SkipComponent``.

    Attributes:
        minor (int): The minor version of RHEL 8.

    Raises:
        SkipComponent: When RHEL version is not RHEL8.
    """
    def __init__(self, rhel):
        super(IsRhel8, self).__init__(rhel, 8)


@component(RedHatRelease)
class IsRhel9(IsRhel):
    """
    This component uses ``RedhatRelease`` combiner
    to determine RHEL version. It checks if RHEL9, if not
    RHEL9 it raises ``SkipComponent``.

    Attributes:
        minor (int): The minor version of RHEL 9.

    Raises:
        SkipComponent: When RHEL version is not RHEL9.
    """
    def __init__(self, rhel):
        super(IsRhel9, self).__init__(rhel, 9)



import doctest

from insights.parsers import ls_run_systemd
from insights.parsers.ls_run_systemd import LsRunSystemd
from insights.tests import context_wrap


# command output is shortened
LS_RUN_SYSTEMD = """
/run/systemd:
total 0
drwxr-xr-x. 16 0 0  400 May 25 09:03 .
drwxr-xr-x. 33 0 0  940 May 25 09:03 ..
srwx------.  1 0 0    0 May 25 09:03 cgroups-agent
srw-------.  1 0 0    0 May 25 09:03 coredump
drwxr-xr-x.  4 0 0  140 May 25 09:03 generator
d---------.  3 0 0  160 May 25 09:03 inaccessible
srwxrwxrwx.  1 0 0    0 May 25 09:03 notify
srwxrwxrwx.  1 0 0    0 May 25 09:03 private
drwxr-xr-x.  2 0 0   40 May 25 09:03 system
drwxr-xr-x.  2 0 0  100 May 25 09:17 transient

/run/systemd/generator:
total 12
drwxr-xr-x.  4 0 0 140 May 25 09:03 .
drwxr-xr-x. 16 0 0 400 May 25 09:03 ..
-rw-r--r--.  1 0 0 254 May 25 09:03 boot.mount
-rw-r--r--.  1 0 0 230 May 25 09:03 dev-mapper-rhel\x2dswap.swap
drwxr-xr-x.  2 0 0  80 May 25 09:03 local-fs.target.requires
-rw-r--r--.  1 0 0 217 May 25 09:03 -.mount
drwxr-xr-x.  2 0 0  60 May 25 09:03 swap.target.requires

/run/systemd/generator/local-fs.target.requires:
total 8
drwxr-xr-x. 2 0 0  80 May 25 09:03 .
drwxr-xr-x. 4 0 0 140 May 25 09:03 ..
-rw-r--r--. 1 0 0 254 May 25 09:03 boot.mount
-rw-r--r--. 1 0 0 217 May 25 09:03 -.mount

/run/systemd/generator/swap.target.requires:
total 4
drwxr-xr-x. 2 0 0  60 May 25 09:03 .
drwxr-xr-x. 4 0 0 140 May 25 09:03 ..
-rw-r--r--. 1 0 0 230 May 25 09:03 dev-mapper-rhel\x2dswap.swap

/run/systemd/system:
total 0
drwxr-xr-x.  2 0 0  40 May 25 09:03 .
drwxr-xr-x. 16 0 0 400 May 25 09:03 ..

/run/systemd/transient:
total 12
drwxr-xr-x.  2 0 0 100 May 25 09:17 .
drwxr-xr-x. 16 0 0 400 May 25 09:03 ..
-rw-r--r--.  1 0 0 275 May 25 09:04 session-6.scope
-rw-r--r--.  1 0 0 275 May 25 09:17 session-7.scope
-rw-r--r--.  1 0 0 275 May 25 09:17 session-8.scope
""".strip()


def test_ls_run_systemd():
    run_systemd = LsRunSystemd(context_wrap(LS_RUN_SYSTEMD))
    assert run_systemd
    assert len(run_systemd.listings) == 6
    assert run_systemd.dirs_of("/run/systemd/generator") == [
        '.', '..', 'local-fs.target.requires', 'swap.target.requires'
    ]
    assert run_systemd.files_of("/run/systemd/generator") == [
        'boot.mount', 'dev-mapper-rhel-swap.swap', '-.mount'
    ]
    assert run_systemd.listing_of("/run/systemd/generator")['-.mount'] == {
        'date': 'May 25 09:03', 'dir': '/run/systemd/generator', 'group': '0', 'links': 1,
        'name': '-.mount', 'owner': '0', 'perms': 'rw-r--r--.',
        'raw_entry': '-rw-r--r--.  1 0 0 217 May 25 09:03 -.mount', 'size': 217, 'type': '-'
    }


def test_ls_osroot_doc_examples():
    env = {
        'ls_run_systemd': LsRunSystemd(context_wrap(LS_RUN_SYSTEMD))
    }
    failed, total = doctest.testmod(ls_run_systemd, globs=env)
    assert failed == 0



"""
GrubEnv - file ``/boot/grub2/grubenv``
======================================

This parser reads the GRUB environment block file. The file is laid out in a
key=value format similar to an ini file except it doesn't have any headers.

The parser stores the key/value pairs in itself which inherits a dict. This
file is only used in Grub 2, but in RHEL8 with BLS being the default. There
were several variables added that are referenced in the
``/boot/loader/entries/*.conf`` files.
"""
from insights import get_active_lines, parser, Parser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.grubenv)
class GrubEnv(Parser, dict):
    """
    Parses the /boot/grub2/grubenv file and returns a dict
    of the grubenv variables.

    Sample output of the file::

        saved_entry=295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64
        kernelopts=root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200
        boot_success=0
        boot_indeterminate=2
        tuned_params=transparent_hugepages=never
        tuned_initrd=

    Attributes:
        has_kernelopts (bool): Returns True/False depending on if kernelopts key is in the dict.
        kernelopts (bool): Returns the string of kernelopts from the dict.
        has_tuned_params (str): Returns True/False depending of if the tuned_params key is in the dict.
        tuned_params (str): Returns the string of tuned_params from the dict.

    Examples:
        >>> type(grubenv)
        <class 'insights.parsers.grubenv.GrubEnv'>
        >>> grubenv.has_kernelopts
        True
        >>> grubenv.kernelopts
        'root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200'
        >>> grubenv.has_tuned_params
        True
        >>> grubenv.tuned_params
        'transparent_hugepages=never'
        >>> grubenv['saved_entry']
        '295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64'
    """
    def __init__(self, context):
        super(GrubEnv, self).__init__(context)

    def parse_content(self, content):
        if not content:
            raise SkipException("Empty output.")

        data = dict()
        for line in get_active_lines(content):
            if "=" not in line:
                continue

            key, value = line.split("=", 1)

            # Some keys can have empty values, so just skip them.
            if not value:
                continue

            data[key] = value

        if not data:
            raise SkipException("No parsed data.")

        self.update(data)

    @property
    def has_kernelopts(self):
        return "kernelopts" in self

    @property
    def kernelopts(self):
        return self.get("kernelopts", "")

    @property
    def has_tuned_params(self):
        return "tuned_params" in self

    @property
    def tuned_params(self):
        return self.get("tuned_params", "")



"""
LsRunSystemd - command ``ls -lanRL /run/systemd``
=================================================

The ``ls -lanRL /run/systemd`` command provides information for the listing of the ``/run/systemd`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Sample output of ``ls -lanRL /run/systemd`` command is::

    /run/systemd:
    total 0
    drwxr-xr-x. 16 0 0  400 May 25 09:03 .
    drwxr-xr-x. 33 0 0  940 May 25 09:03 ..
    srwx------.  1 0 0    0 May 25 09:03 cgroups-agent
    srw-------.  1 0 0    0 May 25 09:03 coredump
    drwxr-xr-x.  4 0 0  140 May 25 09:03 generator
    d---------.  3 0 0  160 May 25 09:03 inaccessible
    srwxrwxrwx.  1 0 0    0 May 25 09:03 notify
    srwxrwxrwx.  1 0 0    0 May 25 09:03 private
    drwxr-xr-x.  2 0 0   40 May 25 09:03 system
    drwxr-xr-x.  2 0 0  100 May 25 09:17 transient

    /run/systemd/generator:
    total 12
    drwxr-xr-x.  4 0 0 140 May 25 09:03 .
    drwxr-xr-x. 16 0 0 400 May 25 09:03 ..
    -rw-r--r--.  1 0 0 254 May 25 09:03 boot.mount
    -rw-r--r--.  1 0 0 230 May 25 09:03 dev-mapper-rhel\x2dswap.swap
    drwxr-xr-x.  2 0 0  80 May 25 09:03 local-fs.target.requires
    -rw-r--r--.  1 0 0 217 May 25 09:03 -.mount
    drwxr-xr-x.  2 0 0  60 May 25 09:03 swap.target.requires

    /run/systemd/generator/local-fs.target.requires:
    total 8
    drwxr-xr-x. 2 0 0  80 May 25 09:03 .
    drwxr-xr-x. 4 0 0 140 May 25 09:03 ..
    -rw-r--r--. 1 0 0 254 May 25 09:03 boot.mount
    -rw-r--r--. 1 0 0 217 May 25 09:03 -.mount

    /run/systemd/generator/swap.target.requires:
    total 4
    drwxr-xr-x. 2 0 0  60 May 25 09:03 .
    drwxr-xr-x. 4 0 0 140 May 25 09:03 ..
    -rw-r--r--. 1 0 0 230 May 25 09:03 dev-mapper-rhel\x2dswap.swap

    /run/systemd/system:
    total 0
    drwxr-xr-x.  2 0 0  40 May 25 09:03 .
    drwxr-xr-x. 16 0 0 400 May 25 09:03 ..

    /run/systemd/transient:
    total 12
    drwxr-xr-x.  2 0 0 100 May 25 09:17 .
    drwxr-xr-x. 16 0 0 400 May 25 09:03 ..
    -rw-r--r--.  1 0 0 275 May 25 09:04 session-6.scope
    -rw-r--r--.  1 0 0 275 May 25 09:17 session-7.scope
    -rw-r--r--.  1 0 0 275 May 25 09:17 session-8.scope

Examples:
    >>> type(ls_run_systemd)
    <class 'insights.parsers.ls_run_systemd.LsRunSystemd'>
    >>> ls_run_systemd.files_of("/run/systemd/generator") == ['boot.mount', 'dev-mapper-rhel-swap.swap', '-.mount']
    True
    >>> ls_run_systemd.dir_entry("/run/systemd/generator", "-.mount")["perms"]
    'rw-r--r--.'
"""

from insights import parser, CommandParser, FileListing
from insights.specs import Specs


@parser(Specs.ls_run_systemd)
class LsRunSystemd(CommandParser, FileListing):
    """Parses output of ``ls -lanRL /run/systemd`` command."""
    pass



from insights.parsers import cciss
from insights.tests import context_wrap


CCISS = '''
cciss0: HP Smart Array P220i Controller
Board ID: 0x3355103c
Firmware Version: 3.42
IRQ: 82
Logical drives: 1
Sector size: 8192
Current Q depth: 0
Current # commands on controller: 0
Max Q depth since init: 84
Max # commands on controller since init: 111
Max SG entries since init: 128
Sequential access devices: 0

cciss/c0d0:  299.96GB   RAID 1(1+0)
'''


def test_get_cciss():
    cciss_info = cciss.Cciss(context_wrap(CCISS, path='/proc/devices/cciss0'))

    assert cciss_info.data['cciss0'] == 'HP Smart Array P220i Controller'
    assert cciss_info.data['Board ID'] == "0x3355103c"
    assert cciss_info.data['Firmware Version'] == '3.42'
    assert cciss_info.data['IRQ'] == '82'
    assert cciss_info.data['Logical drives'] == '1'
    assert cciss_info.data['Sector size'] == '8192'
    assert cciss_info.data['Current Q depth'] == '0'
    assert cciss_info.data['Current # commands on controller'] == '0'
    assert cciss_info.data['Max Q depth since init'] == '84'
    assert cciss_info.data['Max # commands on controller since init'] == '111'
    assert cciss_info.data['Max SG entries since init'] == '128'
    assert cciss_info.data['Sequential access devices'] == '0'
    assert cciss_info.data['cciss/c0d0'] == '299.96GB   RAID 1(1+0)'
    assert cciss_info.firmware_version == '3.42'
    assert cciss_info.model == 'HP Smart Array P220i Controller'



import doctest
import pytest

from insights.parsers import smt, SkipException
from insights.parsers.smt import CpuSMTActive, CpuCoreOnline, CpuSiblings
from insights.tests import context_wrap


def test_cpu_smt_active():
    with pytest.raises(SkipException):
        CpuSMTActive(context_wrap(""))

    p = CpuSMTActive(context_wrap("1"))
    assert p.on
    p = CpuSMTActive(context_wrap("0"))
    assert not p.on


def test_cpu_core_online():
    with pytest.raises(SkipException):
        CpuCoreOnline(context_wrap(""))

    path = "/sys/devices/system/cpu/cpu{0}/online"
    p = CpuCoreOnline(context_wrap("0", path=path.format(0)))
    assert p.core_id == 0
    assert not p.on
    assert repr(p) == "[Core 0: Offline]"
    p = CpuCoreOnline(context_wrap("1", path=path.format(1)))
    assert p.core_id == 1
    assert p.on
    assert repr(p) == "[Core 1: Online]"


def test_cpu_siblings():
    with pytest.raises(SkipException):
        CpuSiblings(context_wrap(""))

    path = "/sys/devices/system/cpu/cpu{0}/topology/thread_siblings_list"
    p = CpuSiblings(context_wrap("0,2", path=path.format(0)))
    assert p.core_id == 0
    assert p.siblings == [0, 2]
    assert repr(p) == "[Core 0 Siblings: [0, 2]]"
    p = CpuSiblings(context_wrap("1-3", path=path.format(3)))
    assert p.core_id == 3
    assert p.siblings == [1, 2, 3]
    assert repr(p) == "[Core 3 Siblings: [1, 2, 3]]"
    p = CpuSiblings(context_wrap("1", path=path.format(1)))
    assert p.core_id == 1
    assert p.siblings == [1]
    assert repr(p) == "[Core 1 Siblings: [1]]"


def test_doc_examples():
    path_cpu_core_online = "/sys/devices/system/cpu/cpu0/online"
    path_cpu_siblings = "/sys/devices/system/cpu/cpu0/topology/thread_siblings_list"

    env = {
        "cpu_smt": CpuSMTActive(context_wrap("1")),
        "cpu_core": CpuCoreOnline(context_wrap("1", path=path_cpu_core_online)),
        "cpu_siblings": CpuSiblings(context_wrap("0,2", path=path_cpu_siblings))
    }
    failed, total = doctest.testmod(smt, globs=env)
    assert failed == 0



"""
CpuVulns - files ``/sys/devices/system/cpu/vulnerabilities/*``
==============================================================

Parser to parse the output of files ``/sys/devices/system/cpu/vulnerabilities/*``
"""

from insights import Parser
from insights import parser
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.cpu_vulns)
class CpuVulns(Parser):
    """
    Base class to parse ``/sys/devices/system/cpu/vulnerabilities/*`` files,
    the file content will be stored in a string.

    Sample output for files:
        ``/sys/devices/system/cpu/vulnerabilities/spectre_v1``::
            Mitigation: Load fences

        ``/sys/devices/system/cpu/vulnerabilities/spectre_v2``::
            Vulnerable: Retpoline without IBPB

        ``/sys/devices/system/cpu/vulnerabilities/meltdown``::
            Mitigation: PTI

        ``/sys/devices/system/cpu/vulnerabilities/spec_store_bypass``::
            Mitigation: Speculative Store Bypass disabled

    Examples:
        >>> type(sp_v1)
        <class 'insights.parsers.cpu_vulns.CpuVulns'>
        >>> type(sp_v1) == type(sp_v2) == type(md) == type(ssb)
        True
        >>> sp_v1.value
        'Mitigation: Load fences'
        >>> sp_v2.value
        'Vulnerable: Retpoline without IBPB'
        >>> md.value
        'Mitigation: PTI'
        >>> ssb.value
        'Mitigation: Speculative Store Bypass disabled'

    Attributes:
        value (str): The result parsed

    Raises:
        SkipException: When file content is empty

    """

    def parse_content(self, content):
        if not content:
            raise SkipException("Input content is empty")
        self.value = content[0]



from insights.parsr import Literal


def test_literal():
    p = Literal("123")
    assert p("123") == "123"


def test_literal_value():
    p = Literal("true", value=True)
    assert p("true") is True


def test_literal_ignore_case():
    p = Literal("true", ignore_case=True)
    assert p("TrUe") == "TrUe"


def test_literal_value_ignore_case():
    p = Literal("true", value=True, ignore_case=True)
    assert p("TRUE") is True



from insights.parsers.ls_systemd import LsEtcSystemd, LsUsrLibSystemd
from insights.parsers import ls_systemd
from insights.tests import context_wrap
import doctest

ETC_SYSTEMD = """
/etc/systemd:
total 48
drwxr-xr-x   10 501  20   320 Feb 23  2021 .
drwxr-xr-x  129 501  20  4128 Nov  7 15:01 ..
-rw-r--r--    1 501  20   720 Jan 16  2021 bootchart.conf
-rw-r--r--    1 501  20   615 Jan 16  2021 coredump.conf
-rw-r--r--    1 501  20   983 Jan 16  2021 journald.conf
-rw-r--r--    1 501  20   957 Jan 16  2021 logind.conf
drwxr-xr-x   39 501  20  1248 Oct 14 04:49 system
-rw-r--r--    1 501  20  1552 Jan 16  2021 system.conf
drwxr-xr-x    2 501  20    64 Jan 16  2021 user
-rw-r--r--    1 501  20  1127 Jan 16  2021 user.conf

/etc/systemd/system:
total 104
drwxr-xr-x  39 501  20  1248 Oct 14 04:49 .
drwxr-xr-x  10 501  20   320 Feb 23  2021 ..
drwxr-xr-x   5 501  20   160 Jun 19  2018 basic.target.wants
-rw-r--r--   1 501  20   657 Apr 16  2021 dbus-org.fedoraproject.FirewallD1.service
-rw-r--r--   1 501  20  1445 Sep 30  2020 dbus-org.freedesktop.NetworkManager.service
-rw-r--r--   1 501  20   353 Sep 30  2020 dbus-org.freedesktop.nm-dispatcher.service
-rw-r--r--   1 501  20   492 Jan 16  2021 default.target
-rw-r--r--   1 501  20   149 Oct 13 02:51 postgresql.service
-r--r--r--   1 501  20   946 Oct 13 03:02 pulpcore-api.service
-r--r--r--   1 501  20   157 Oct 13 03:02 pulpcore-api.socket
-r--r--r--   1 501  20   832 Oct 14 04:49 pulpcore-content.service
-r--r--r--   1 501  20   162 Oct 13 03:02 pulpcore-content.socket
-r--r--r--   1 501  20   813 Oct 13 03:02 pulpcore-resource-manager.service
-r--r--r--   1 501  20   861 Oct 13 03:02 pulpcore-worker@.service
-rw-r--r--   1 501  20   164 May 13  2020 receptor@.service
-rw-r--r--   1 501  20   602 Jan 16  2020 rh-mongodb34-mongod.service
crw-rw-rw-   1  0   0 1,   3 Mar  2 06:03 systemd-timedated.service

/etc/systemd/system/basic.target.wants:
total 24
drwxr-xr-x   5 501  20   160 Jun 19  2018 .
drwxr-xr-x  39 501  20  1248 Oct 14 04:49 ..
-rw-r--r--   1 501  20   657 Apr 16  2021 firewalld.service
-rw-r--r--   1 501  20   284 Jul 28  2021 microcode.service
-rw-r--r--   1 501  20   217 May 22  2020 rhel-dmesg.service

/etc/systemd/user:
total 0
drwxr-xr-x   2 501  20   64 Jan 16  2021 .
drwxr-xr-x  10 501  20  320 Feb 23  2021 ..
"""

ETC_SYSTEMD_EXAMPLE = """
/etc/systemd:
total 48
drwxr-xr-x   10 501  20   320 Feb 23  2021 .
drwxr-xr-x  129 501  20  4128 Nov  7 15:01 ..
-rw-r--r--    1 501  20   720 Jan 16  2021 bootchart.conf
-rw-r--r--    1 501  20   615 Jan 16  2021 coredump.conf
drwxr-xr-x    2 501  20    64 Jan 16  2021 user
-rw-r--r--    1 501  20  1127 Jan 16  2021 user.conf

/etc/systemd/user:
total 0
drwxr-xr-x   2 501  20   64 Jan 16  2021 .
drwxr-xr-x  10 501  20  320 Feb 23  2021 ..
"""

USR_LIB_SYSTEMD = """
/usr/lib/systemd:
total 0
drwxr-xr-x    4 501  20    128 Feb 23  2021 .
dr-xr-xr-x    8 501  20    256 Oct 13 01:34 ..
drwxr-xr-x  404 501  20  12928 Oct 13 02:50 system
drwxr-xr-x   15 501  20    480 Feb 23  2021 user

/usr/lib/systemd/user:
total 104
drwxr-xr-x  15 501  20  480 Feb 23  2021 .
drwxr-xr-x   4 501  20  128 Feb 23  2021 ..
-rw-r--r--   1 501  20  457 Jan 16  2021 basic.target
-rw-r--r--   1 501  20  379 Jan 16  2021 bluetooth.target
-rw-r--r--   1 501  20  414 Jan 16  2021 default.target
-rw-r--r--   1 501  20  499 Jan 16  2021 exit.target
-rw-r--r--   1 501  20  147 May 31  2018 glib-pacrunner.service
-rw-r--r--   1 501  20  354 Jan 16  2021 paths.target
-rw-r--r--   1 501  20  377 Jan 16  2021 printer.target
-rw-r--r--   1 501  20  402 Jan 16  2021 shutdown.target
-rw-r--r--   1 501  20  380 Jan 16  2021 smartcard.target
-rw-r--r--   1 501  20  356 Jan 16  2021 sockets.target
-rw-r--r--   1 501  20  380 Jan 16  2021 sound.target
-rw-r--r--   1 501  20  501 Jan 16  2021 systemd-exit.service
-rw-r--r--   1 501  20  405 Jan 16  2021 timers.target
"""


def test_etc_systemd():
    etc_systemd_perm = LsEtcSystemd(context_wrap(ETC_SYSTEMD))
    assert '/etc/systemd' in etc_systemd_perm
    assert '/etc/systemd/system' in etc_systemd_perm
    assert '/etc/systemd/system/basic.target.wants' in etc_systemd_perm
    assert '/etc/systemd/user' in etc_systemd_perm
    assert etc_systemd_perm.dirs_of('/etc/systemd') == ['.', '..', 'system', 'user']

    basic_target_wants = etc_systemd_perm.files_of('/etc/systemd/system/basic.target.wants')
    assert 'firewalld.service' in basic_target_wants
    assert 'microcode.service' in basic_target_wants
    assert 'rhel-dmesg.service' in basic_target_wants
    assert 'systemd-timedated.service' not in basic_target_wants
    assert len(basic_target_wants) == 3

    assert etc_systemd_perm.listing_of('/etc/systemd')['user.conf'] == {'type': '-', 'perms': 'rw-r--r--', 'links': 1, 'owner': '501', 'group': '20', 'size': 1127, 'date': 'Jan 16  2021', 'name': 'user.conf', 'raw_entry': '-rw-r--r--    1 501  20  1127 Jan 16  2021 user.conf', 'dir': '/etc/systemd'}


def test_usr_lib_systemd():
    usr_lib_systemd_perm = LsUsrLibSystemd(context_wrap(USR_LIB_SYSTEMD))
    assert '/usr/lib/systemd' in usr_lib_systemd_perm
    assert '/usr/lib/systemd/user' in usr_lib_systemd_perm
    assert usr_lib_systemd_perm.dirs_of('/usr/lib/systemd') == ['.', '..', 'system', 'user']

    usr_lib_systemd_use = usr_lib_systemd_perm.files_of('/usr/lib/systemd/user')
    assert 'basic.target' in usr_lib_systemd_use
    assert 'bluetooth.target' in usr_lib_systemd_use
    assert len(usr_lib_systemd_use) == 13

    assert usr_lib_systemd_perm.listing_of('/usr/lib/systemd/user')['timers.target'] == {'type': '-', 'perms': 'rw-r--r--', 'links': 1, 'owner': '501', 'group': '20', 'size': 405, 'date': 'Jan 16  2021', 'name': 'timers.target', 'raw_entry': '-rw-r--r--   1 501  20  405 Jan 16  2021 timers.target', 'dir': '/usr/lib/systemd/user'}


def test_systemd_examples():
    env = {
        'etc_systemd': LsEtcSystemd(context_wrap(ETC_SYSTEMD_EXAMPLE)),
        'usr_lib_systemd': LsUsrLibSystemd(context_wrap(USR_LIB_SYSTEMD))
    }
    failed, total = doctest.testmod(ls_systemd, globs=env)
    assert failed == 0



from insights.parsers.journald_conf import EtcJournaldConf, EtcJournaldConfD, UsrJournaldConfD
from insights.combiners.journald_conf import JournaldConfAll
from insights.tests import context_wrap

# /etc/systemd/journald.conf
JOURNALD_CONF_1 = """
[Journal]
#Storage=auto
Storage=persistent
#Compress=yes
#Seal=yes
Seal=no
#SplitMode=uid
#SyncIntervalSec=5m
#RateLimitInterval=30s
#RateLimitBurst=1000
#SystemMaxUse=
#SystemKeepFree=
#SystemMaxFileSize=
#RuntimeMaxUse=
#RuntimeKeepFree=
#RuntimeMaxFileSize=
#MaxRetentionSec=
#MaxFileSec=1month
#ForwardToSyslog=yes
ForwardToSyslog=no
#ForwardToKMsg=no
#ForwardToConsole=no
#ForwardToWall=yes
#TTYPath=/dev/console
#MaxLevelStore=debug
#MaxLevelSyslog=debug
#MaxLevelKMsg=notice
#MaxLevelConsole=info
#MaxLevelWall=emerg
""".strip()

# /etc/systemd/journald.conf.d/a.conf
JOURNALD_CONF_2 = """
[Journal]
Storage=volatile
Compress=no
RateLimitInterval=10s
# not set: ForwardToConsole=yes
""".strip()

# /usr/lib/systemd/journald.conf.d/a.conf
JOURNALD_CONF_3 = """
[Journal]
Storage=none
Compress=no
ForwardToConsole=yes
""".strip()

# /etc/systemd/journald.conf.d/b.conf
JOURNALD_CONF_4 = """
[Journal]
SplitMode=uid
SyncIntervalSec=20m
MaxLevelWall=emerg
""".strip()

# /usr/lib/systemd/journald.conf.d/b.conf
JOURNALD_CONF_5 = """
[Journal]
Storage=auto
Seal=yes
SyncIntervalSec=5m
""".strip()


# These tests test all the various combinations of file priority and file shadowing.
# Intentionally breaking column width so that each assert is on a single line.


def test_1():
    # smoke test with just journald.conf
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    result = JournaldConfAll(conf1, None, None)
    assert result.active_settings_with_file_name["Storage"] == ("persistent", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert "Compress" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name


def test_2():
    # smoke test with two files and no shadowing
    # a.conf options have higher priority than the options from journald.conf
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_2, path='/etc/systemd/journald.conf.d/a.conf'))
    result = JournaldConfAll(conf1, [conf2], None)
    assert result.active_settings_with_file_name["Storage"] == ("volatile", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value("RateLimitInterval") == "10s"
    assert "ForwardToConsole" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == []
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/etc/systemd/journald.conf.d/a.conf']


def test_3():
    # smoke test with three files - one shadowing and one option key taking higher priority
    # file in /etc shadows the file in /usr, and these options take precedence over journald.conf
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_2, path='/etc/systemd/journald.conf.d/a.conf'))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    result = JournaldConfAll(conf1, [conf2], [conf3])
    assert result.active_settings_with_file_name["Storage"] == ("volatile", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value("RateLimitInterval") == "10s"
    assert "ForwardToConsole" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == ['/usr/lib/systemd/journald.conf.d/a.conf']
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/etc/systemd/journald.conf.d/a.conf']


def test_4():
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    result = JournaldConfAll(conf1, None, [conf3])
    assert result.active_settings_with_file_name["Storage"] == ("none", "/usr/lib/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value_and_file_name("Compress") == ("no", "/usr/lib/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("ForwardToConsole") == "yes"
    assert result.get_active_setting_value_and_file_name("ForwardToConsole") == ("yes", "/usr/lib/systemd/journald.conf.d/a.conf")
    assert "RateLimitInterval" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == []
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/usr/lib/systemd/journald.conf.d/a.conf']


def test_5():
    # as an addition to test_3, there's a new file whose options have higher priority
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_2, path='/etc/systemd/journald.conf.d/a.conf'))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    conf4 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_4, path='/etc/systemd/journald.conf.d/b.conf'))
    result = JournaldConfAll(conf1, [conf2, conf4], [conf3])
    assert result.active_settings_with_file_name["Storage"] == ("volatile", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value_and_file_name("Compress") == ("no", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("RateLimitInterval") == "10s"
    assert result.get_active_setting_value("SplitMode") == "uid"
    assert result.get_active_setting_value("SyncIntervalSec") == "20m"
    assert result.get_active_setting_value("MaxLevelWall") == "emerg"
    assert "ForwardToConsole" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == ['/usr/lib/systemd/journald.conf.d/a.conf']
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/etc/systemd/journald.conf.d/a.conf', '/etc/systemd/journald.conf.d/b.conf']


def test_5_swap():
    # same as test_5, just the order of two reported files is changed (but not their filename)
    #  - tests that the file names, not the reported order, are important to the algorithm
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_2, path='/etc/systemd/journald.conf.d/a.conf'))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    conf4 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_4, path='/etc/systemd/journald.conf.d/b.conf'))
    result = JournaldConfAll(conf1, [conf4, conf2], [conf3])
    assert result.active_settings_with_file_name["Storage"] == ("volatile", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value_and_file_name("Compress") == ("no", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("RateLimitInterval") == "10s"
    assert result.get_active_setting_value("SplitMode") == "uid"
    assert result.get_active_setting_value("SyncIntervalSec") == "20m"
    assert result.get_active_setting_value("MaxLevelWall") == "emerg"
    assert "ForwardToConsole" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == ['/usr/lib/systemd/journald.conf.d/a.conf']
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/etc/systemd/journald.conf.d/a.conf', '/etc/systemd/journald.conf.d/b.conf']


def test_6():
    # same as test_5, /usr/lib/systemd/journald.conf.d/b.conf is shadowed anyway
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_2, path='/etc/systemd/journald.conf.d/a.conf'))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    conf4 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_4, path='/etc/systemd/journald.conf.d/b.conf'))
    conf5 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_5, path='/usr/lib/systemd/journald.conf.d/b.conf'))
    result = JournaldConfAll(conf1, [conf2, conf4], [conf3, conf5])
    assert result.active_settings_with_file_name["Storage"] == ("volatile", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value_and_file_name("Compress") == ("no", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("RateLimitInterval") == "10s"
    assert result.get_active_setting_value("SplitMode") == "uid"
    assert result.get_active_setting_value("SyncIntervalSec") == "20m"
    assert result.get_active_setting_value("MaxLevelWall") == "emerg"
    assert "ForwardToConsole" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == ['/usr/lib/systemd/journald.conf.d/a.conf', '/usr/lib/systemd/journald.conf.d/b.conf']
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/etc/systemd/journald.conf.d/a.conf', '/etc/systemd/journald.conf.d/b.conf']


def test_7():
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_2, path='/etc/systemd/journald.conf.d/a.conf'))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    conf5 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_5, path='/usr/lib/systemd/journald.conf.d/b.conf'))
    result = JournaldConfAll(conf1, [conf2], [conf3, conf5])
    assert result.active_settings_with_file_name["Storage"] == ("auto", "/usr/lib/systemd/journald.conf.d/b.conf")
    assert result.get_active_setting_value("Seal") == "yes"
    assert result.get_active_setting_value_and_file_name("Seal") == ("yes", "/usr/lib/systemd/journald.conf.d/b.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value_and_file_name("Compress") == ("no", "/etc/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("RateLimitInterval") == "10s"
    assert result.get_active_setting_value("SyncIntervalSec") == "5m"
    assert "ForwardToConsole" not in result.active_settings_with_file_name
    assert "MaxLevelWall" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == ['/usr/lib/systemd/journald.conf.d/a.conf']
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/etc/systemd/journald.conf.d/a.conf', '/usr/lib/systemd/journald.conf.d/b.conf']


def test_8():
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    conf5 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_5, path='/usr/lib/systemd/journald.conf.d/b.conf'))
    result = JournaldConfAll(conf1, None, [conf3, conf5])
    assert result.active_settings_with_file_name["Storage"] == ("auto", "/usr/lib/systemd/journald.conf.d/b.conf")
    assert result.get_active_setting_value("Seal") == "yes"
    assert result.get_active_setting_value_and_file_name("Seal") == ("yes", "/usr/lib/systemd/journald.conf.d/b.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value_and_file_name("Compress") == ("no", "/usr/lib/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("ForwardToConsole") == "yes"
    assert result.get_active_setting_value_and_file_name("ForwardToConsole") == ("yes", "/usr/lib/systemd/journald.conf.d/a.conf")
    assert result.get_active_setting_value("SyncIntervalSec") == "5m"
    assert "RateLimitInterval" not in result.active_settings_with_file_name
    assert "MaxLevelWall" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == []
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf', '/usr/lib/systemd/journald.conf.d/a.conf', '/usr/lib/systemd/journald.conf.d/b.conf']


def test_9():
    # empty files in /etc and /usr, same as test_1
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap("", path='/etc/systemd/journald.conf.d/a.conf'))
    conf3 = UsrJournaldConfD(context_wrap("", path='/usr/lib/systemd/journald.conf.d/a.conf'))
    conf4 = EtcJournaldConfD(context_wrap("", path='/etc/systemd/journald.conf.d/b.conf'))
    conf5 = UsrJournaldConfD(context_wrap("", path='/usr/lib/systemd/journald.conf.d/b.conf'))
    result = JournaldConfAll(conf1, [conf2, conf4], [conf3, conf5])
    assert result.active_settings_with_file_name["Storage"] == ("persistent", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert "Compress" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == ['/usr/lib/systemd/journald.conf.d/a.conf', '/usr/lib/systemd/journald.conf.d/b.conf']
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf']  # other files are empty and effectively not used


def test_10():
    # feeding it None data on purpose
    # empty/null files in /etc and /usr, same as test_1
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(None, path=None))
    conf3 = UsrJournaldConfD(context_wrap(None, path='/usr/lib/systemd/journald.conf.d/a.conf'))
    conf4 = EtcJournaldConfD(context_wrap("", path=None))
    conf5 = UsrJournaldConfD(context_wrap(None, path=None))
    result = JournaldConfAll(conf1, [conf2, conf4], [conf3, conf5])
    assert result.active_settings_with_file_name["Storage"] == ("persistent", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert "Compress" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == []
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf']  # a.conf is empty


def test_11():
    # Even if the path is None (in case Parser somehow fails to get it), make sure priorities work.
    #  (shadowing in this case, as None is taken as a file name; this is already operation in
    #   undefined/incorrect conditions)
    # This test will also help in frontend debugging so that it can work correctly when no path is
    #  given.
    # empty/null files in /etc and /usr, same as test_3
    conf1 = EtcJournaldConf(context_wrap(JOURNALD_CONF_1, path='/etc/systemd/journald.conf'))
    conf2 = EtcJournaldConfD(context_wrap(JOURNALD_CONF_2, path=None))
    conf3 = UsrJournaldConfD(context_wrap(JOURNALD_CONF_3, path=None))
    result = JournaldConfAll(conf1, [conf2], [conf3])
    assert result.active_settings_with_file_name["Storage"] == ("volatile", None)
    assert result.get_active_setting_value("Seal") == "no"
    assert result.get_active_setting_value_and_file_name("Seal") == ("no", "/etc/systemd/journald.conf")
    assert result.get_active_setting_value("Compress") == "no"
    assert result.get_active_setting_value("RateLimitInterval") == "10s"
    assert "ForwardToConsole" not in result.active_settings_with_file_name
    assert "SplitMode" not in result.active_settings_with_file_name
    assert result.files_shadowed_not_used == []
    assert result.files_used_priority_order == ['/etc/systemd/journald.conf']



from insights.parsers.installed_rpms import InstalledRpms
from insights.combiners import ansible_info
from insights.combiners.ansible_info import (
    AnsibleInfo, ANSIBLE_AUTOMATION_CONTROLLER_PKG, ANSIBLE_CATALOG_WORKER_PKG, ANSIBLE_TOWER_PKG,
    ANSIBLE_AUTOMATION_HUB_PKG)
from insights.tests import context_wrap
import doctest

TOWER_RPM = ANSIBLE_TOWER_PKG + "-1.0.0-1"
AUTO_CONTROLLER_RPM = ANSIBLE_AUTOMATION_CONTROLLER_PKG + "-1.0.1-1"
CATALOG_WORKER_RPM = ANSIBLE_CATALOG_WORKER_PKG + "-1.0.2-1"
HUB_RPM = ANSIBLE_AUTOMATION_HUB_PKG + "-1.0.3-1"
ALL_RPMS = '''
{controller}
{cworker}
{tower}
{hub}
'''.format(
    controller=AUTO_CONTROLLER_RPM,
    cworker=CATALOG_WORKER_RPM,
    tower=TOWER_RPM,
    hub=HUB_RPM).strip()


def test_ansible_info_all():
    rpms = InstalledRpms(context_wrap(ALL_RPMS))
    comb = AnsibleInfo(rpms)
    assert comb is not None
    assert comb.is_tower
    assert comb.tower_version == '1.0.0'
    assert comb[ANSIBLE_TOWER_PKG].nvr == TOWER_RPM
    assert comb[ANSIBLE_AUTOMATION_CONTROLLER_PKG].nvr == AUTO_CONTROLLER_RPM
    assert comb.is_controller
    assert comb.controller_version == '1.0.0'
    assert comb.is_hub
    assert comb.hub_version == '1.0.3'
    assert comb[ANSIBLE_AUTOMATION_HUB_PKG].nvr == HUB_RPM
    assert comb.is_catalog_worker
    assert comb.catalog_worker_version == '1.0.2'
    assert comb[ANSIBLE_CATALOG_WORKER_PKG].nvr == CATALOG_WORKER_RPM


def test_ansible_info_tower():
    rpms = InstalledRpms(context_wrap(TOWER_RPM))
    comb = AnsibleInfo(rpms)
    assert comb.is_tower
    assert comb.is_controller
    assert not comb.is_hub
    assert not comb.is_catalog_worker
    assert comb.tower_version == '1.0.0'
    assert comb.controller_version == '1.0.0'
    assert comb.hub_version is None
    assert comb.catalog_worker_version is None


def test_ansible_info_auto_controller():
    rpms = InstalledRpms(context_wrap(AUTO_CONTROLLER_RPM))
    comb = AnsibleInfo(rpms)
    assert not comb.is_tower
    assert comb.is_controller
    assert not comb.is_hub
    assert not comb.is_catalog_worker
    assert comb.tower_version is None
    assert comb.controller_version == '1.0.1'
    assert comb.hub_version is None
    assert comb.catalog_worker_version is None


def test_ansible_info_docs():
    rpms = InstalledRpms(context_wrap(TOWER_RPM))
    comb = AnsibleInfo(rpms)
    env = {'info': comb}
    failed, total = doctest.testmod(ansible_info, globs=env)
    assert failed == 0



from insights.combiners.user_namespaces import UserNamespaces
from insights.parsers.cmdline import CmdLine
from insights.parsers.grub_conf import Grub2Config
from insights.tests import context_wrap

ENABLE_TOK_A = '''
user_namespaces.enable=1
'''.strip()  # noqa

ENABLE_TOK_B = '''
user-namespaces.enable=1
'''.strip()  # noqa

CMDLINE = '''
BOOT_IMAGE=/vmlinuz-3.10.0-514.6.1.el7.x86_64 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap {0}
'''.strip()  # noqa

GRUB2_CONF = '''
### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.16.1.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {{
        load_video
        set gfxpayload=keep
        insmod gzio
        insmod part_gpt
        insmod xfs
        set root='hd0,gpt2'
        if [ x$feature_platform_search_hint = xy ]; then
          search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
        else
          search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
        fi
        linuxefi /vmlinuz-3.10.0-514.16.1.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8 {0}
        initrdefi /initramfs-3.10.0-514.16.1.el7.x86_64.img
}}
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.10.2.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {{
        load_video
        set gfxpayload=keep
        insmod gzio
        insmod part_gpt
        insmod xfs
        set root='hd0,gpt2'
        if [ x$feature_platform_search_hint = xy ]; then
          search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
        else
          search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
        fi
        linuxefi /vmlinuz-3.10.0-514.10.2.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8 {1}
        initrdefi /initramfs-3.10.0-514.10.2.el7.x86_64.img
}}
'''  # noqa

MENUENTRY_0 = '''
'Red Hat Enterprise Linux Server (3.10.0-514.16.1.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586'
'''.strip()  # noqa

MENUENTRY_1 = '''
'Red Hat Enterprise Linux Server (3.10.0-514.10.2.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586'
'''.strip()  # noqa

CASES = [
        # noqa
        # |-- provided --|  |---- expected results ---|
        # ((cmdline, grub), (enabled, enabled_configs))
        # Not enabled, no grub data
        ((CMDLINE.format(''), None), (False, [])),
        # Not enabled, not enabled in grub
        ((CMDLINE.format(''), GRUB2_CONF.format('', '')), (False, [])),
        # Not enabled, but enabled in menuentry 1
        ((CMDLINE.format(''), GRUB2_CONF.format('', ENABLE_TOK_A)),
         (False, [MENUENTRY_1])),
        # Enabled, no grub data
        ((CMDLINE.format(ENABLE_TOK_A), None), (True, [])),
        # Enabled, but not enabled in grub
        ((CMDLINE.format(ENABLE_TOK_A), GRUB2_CONF.format('', '')),
         (True, [])),
        # Enabled, enabled in menuentry 0
        ((CMDLINE.format(ENABLE_TOK_A), GRUB2_CONF.format(ENABLE_TOK_A, '')),
         (True, [MENUENTRY_0])),
        # Dash syntax, rather than underscore
        ((CMDLINE.format(ENABLE_TOK_B), GRUB2_CONF.format(ENABLE_TOK_B, '')),
         (True, [MENUENTRY_0]))
]


def test_integration():
    for case in CASES:
        context = dict()
        context[CmdLine] = CmdLine(context_wrap(case[0][0]))
        if case[0][1] is not None:
            context[Grub2Config] = Grub2Config(context_wrap(case[0][1]))

        un = UserNamespaces(context.get(CmdLine), context.get(Grub2Config))
        assert un.enabled() == case[1][0]
        assert un.enabled_configs() == case[1][1]



from glob import glob
from insights.client.archive import InsightsArchive
from insights.client.connection import InsightsConnection
from insights.client.constants import InsightsConstants as constants
from insights.client.utilities import os_release_info
from logging import getLogger
from re import findall
from sys import exit
import tempfile
from insights.util.subproc import call
import os
import os.path
import pkgutil
import six
import sys
import yaml

# Since XPath expression is not supported by the ElementTree in Python 2.6,
# import insights.contrib.ElementTree when running python is prior to 2.6 for compatibility.
# Script insights.contrib.ElementTree is the same with xml.etree.ElementTree in Python 2.7.14
# Otherwise, import defusedxml.ElementTree to avoid XML vulnerabilities,
# if dependency not installed import xml.etree.ElementTree instead.
if sys.version_info[0] == 2 and sys.version_info[1] <= 6:
    import insights.contrib.ElementTree as ET
else:
    try:
        import defusedxml.ElementTree as ET
    except:
        import xml.etree.ElementTree as ET

NONCOMPLIANT_STATUS = 2
COMPLIANCE_CONTENT_TYPE = 'application/vnd.redhat.compliance.something+tgz'
POLICY_FILE_LOCATION = '/usr/share/xml/scap/ssg/content/'
SCAP_DATASTREAMS_PATH = '/usr/share/xml/scap/'
SSG_PACKAGE = 'scap-security-guide'
REQUIRED_PACKAGES = [SSG_PACKAGE, 'openscap-scanner', 'openscap']

# SSG versions that need the <version> in XML repaired
VERSIONS_FOR_REPAIR = '0.1.18 0.1.19 0.1.21 0.1.25'.split()
SNIPPET_TO_FIX = '<version>0.9</version>'

logger = getLogger(__name__)


class ComplianceClient:
    def __init__(self, config):
        self.config = config
        self.conn = InsightsConnection(config)
        self.archive = InsightsArchive(config)
        self._ssg_version = None

    def oscap_scan(self):
        self.inventory_id = self._get_inventory_id()
        self._assert_oscap_rpms_exist()
        initial_profiles = self.get_initial_profiles()
        matching_os_profiles = self.get_profiles_matching_os()
        profiles = self.profile_union_by_ref_id(matching_os_profiles, initial_profiles)
        if not profiles:
            logger.error("System is not associated with any profiles. Assign profiles using the Compliance web UI.\n")
            exit(constants.sig_kill_bad)

        archive_dir = self.archive.create_archive_dir()
        results_need_repair = self.results_need_repair()

        for profile in profiles:
            tailoring_file = self.download_tailoring_file(profile)
            results_file = self._results_file(archive_dir, profile)
            self.run_scan(
                profile['attributes']['ref_id'],
                self.find_scap_policy(profile['attributes']['ref_id']),
                results_file,
                tailoring_file_path=tailoring_file
            )
            if self.config.obfuscate:
                tree = ET.parse(results_file)
                # Retrieve the list of xpaths that need to be obfuscated
                xpaths = yaml.load(pkgutil.get_data('insights', 'compliance_obfuscations.yaml'), Loader=yaml.SafeLoader)
                # Obfuscate IP addresses in the XCCDF report
                self.obfuscate(tree, xpaths['obfuscate'])
                if self.config.obfuscate_hostname:
                    # Obfuscate the hostname in the XCCDF report
                    self.obfuscate(tree, xpaths['obfuscate_hostname'])
                # Overwrite the results file with the obfuscations
                tree.write(results_file)
            if results_need_repair:
                self.repair_results(results_file)
            if tailoring_file:
                os.remove(tailoring_file)

        return self.archive.create_tar_file(), COMPLIANCE_CONTENT_TYPE

    def download_tailoring_file(self, profile):
        if ('tailored' not in profile['attributes'] or profile['attributes']['tailored'] is False or
                ('os_minor_version' in profile['attributes'] and profile['attributes']['os_minor_version'] != self.os_minor_version())):
            return None

        # Download tailoring file to pass as argument to run_scan
        logger.debug(
            "Policy {0} is a tailored policy. Starting tailoring file download...".format(profile['attributes']['ref_id'])
        )
        tailoring_file_path = tempfile.mkstemp(
            prefix='oscap_tailoring_file-{0}.'.format(profile['attributes']['ref_id']),
            suffix='.xml',
            dir='/var/tmp'
        )[1]
        response = self.conn.session.get(
            "https://{0}/compliance/profiles/{1}/tailoring_file".format(self.config.base_url, profile['id'])
        )
        logger.debug("Response code: {0}".format(response.status_code))
        if response.content is None:
            logger.info("Problem downloading tailoring file for {0} to {1}".format(profile['attributes']['ref_id'], tailoring_file_path))
            return None

        with open(tailoring_file_path, mode="w+b") as f:
            f.write(response.content)
            logger.info("Saved tailoring file for {0} to {1}".format(profile['attributes']['ref_id'], tailoring_file_path))

        logger.debug("Policy {0} tailoring file download finished".format(profile['attributes']['ref_id']))

        return tailoring_file_path

    def get_profiles(self, search):
        response = self.conn.session.get("https://{0}/compliance/profiles".format(self.config.base_url),
                                         params={'search': search, 'relationships': 'false'})
        logger.debug("Content of the response: {0} - {1}".format(response,
                                                                 response.json()))
        if response.status_code == 200:
            return (response.json().get('data') or [])
        else:
            return []

    def get_initial_profiles(self):
        return self.get_profiles('system_ids={0} canonical=false external=false'.format(self.inventory_id))

    def get_profiles_matching_os(self):
        return self.get_profiles('system_ids={0} canonical=false os_minor_version={1}'.format(self.inventory_id, self.os_minor_version()))

    def profile_union_by_ref_id(self, prioritized_profiles, merged_profiles):
        profiles = dict((p['attributes']['ref_id'], p) for p in merged_profiles)
        profiles.update(dict((p['attributes']['ref_id'], p) for p in prioritized_profiles))

        return list(profiles.values())

    def os_release(self):
        _, version = os_release_info()
        return version

    def os_major_version(self):
        return findall("^[6-9]", self.os_release())[0]

    def os_minor_version(self):
        return findall("\d+$", self.os_release())[0]

    def profile_files(self):
        return glob("{0}*rhel{1}*.xml".format(POLICY_FILE_LOCATION, self.os_major_version()))

    def find_scap_policy(self, profile_ref_id):
        grepcmd = 'grep -H ' + profile_ref_id + ' ' + ' '.join(self.profile_files())
        if not six.PY3:
            grepcmd = grepcmd.encode()
        rc, grep = call(grepcmd, keep_rc=True)
        if rc:
            logger.error('XML profile file not found matching ref_id {0}\n{1}\n'.format(profile_ref_id, grep))
            return None
        filenames = findall(SCAP_DATASTREAMS_PATH + '.+xml', grep)
        if not filenames:
            logger.error('No XML profile files found matching ref_id {0}\n{1}\n'.format(profile_ref_id, ' '.join(filenames)))
            exit(constants.sig_kill_bad)
        return filenames[0]

    def build_oscap_command(self, profile_ref_id, policy_xml, output_path, tailoring_file_path):
        command = 'oscap xccdf eval --profile ' + profile_ref_id
        if tailoring_file_path:
            command += ' --tailoring-file ' + tailoring_file_path
        command += ' --results ' + output_path + ' ' + policy_xml
        return command

    def run_scan(self, profile_ref_id, policy_xml, output_path, tailoring_file_path=None):
        if policy_xml is None:
            return
        logger.info('Running scan for {0}... this may take a while'.format(profile_ref_id))
        env = os.environ.copy()
        env.update({'TZ': 'UTC'})
        oscap_command = self.build_oscap_command(profile_ref_id, policy_xml, output_path, tailoring_file_path)
        if not six.PY3:
            oscap_command = oscap_command.encode()
        rc, oscap = call(oscap_command, keep_rc=True, env=env)
        if rc and rc != NONCOMPLIANT_STATUS:
            logger.error('Scan failed')
            logger.error(oscap)
            exit(constants.sig_kill_bad)

    @property
    def ssg_version(self):
        if not self._ssg_version:
            self._ssg_version = self.get_ssg_version()
        return self._ssg_version

    def get_ssg_version(self):
        rpmcmd = 'rpm -qa --qf "%{VERSION}" ' + SSG_PACKAGE
        if not six.PY3:
            rpmcmd = rpmcmd.encode()

        rc, ssg_version = call(rpmcmd, keep_rc=True)
        if rc:
            logger.warning('Tried determinig SSG version but failed: {0}.\n'.format(ssg_version))
            return

        logger.info('System uses SSG version %s', ssg_version)
        return ssg_version

    # Helper function that traverses through the XCCDF report and replaces the content of each
    # matching xpath with an empty string
    def obfuscate(self, tree, xpaths):
        for xpath in xpaths:
            for node in tree.findall(xpath):
                node.text = ''

    def results_need_repair(self):
        return self.ssg_version in VERSIONS_FOR_REPAIR

    def repair_results(self, results_file):
        if not os.path.isfile(results_file):
            return
        if not self.ssg_version:
            logger.warning("Couldn't repair SSG version in results file %s", results_file)
            return

        results_file_in = '{0}.in'.format(results_file)
        os.rename(results_file, results_file_in)

        with open(results_file_in, 'r') as in_file:
            with open(results_file, 'w') as out_file:
                is_repaired = self._repair_ssg_version_in_results(
                    in_file, out_file, self.ssg_version
                )

        os.remove(results_file_in)
        if is_repaired:
            logger.debug('Repaired version in results file %s', results_file)
        return is_repaired

    def _repair_ssg_version_in_results(self, in_file, out_file, ssg_version):
        replacement = '<version>{0}</version>'.format(ssg_version)
        is_repaired = False
        for line in in_file:
            if is_repaired or SNIPPET_TO_FIX not in line:
                out_file.write(line)
            else:
                out_file.write(line.replace(SNIPPET_TO_FIX, replacement))
                is_repaired = True
                logger.debug(
                    'Substituted "%s" with "%s" in %s',
                    SNIPPET_TO_FIX, replacement, out_file.name
                )

        return is_repaired

    def _assert_oscap_rpms_exist(self):
        rpmcmd = 'rpm -qa ' + ' '.join(REQUIRED_PACKAGES)
        if not six.PY3:
            rpmcmd = rpmcmd.encode()
        rc, rpm = call(rpmcmd, keep_rc=True)
        if rc:
            logger.error('Tried running rpm -qa but failed: {0}.\n'.format(rpm))
            exit(constants.sig_kill_bad)
        else:
            if len(rpm.strip().split('\n')) < len(REQUIRED_PACKAGES):
                logger.error('Missing required packages for compliance scanning. Please ensure the following packages are installed: {0}\n'.format(', '.join(REQUIRED_PACKAGES)))
                exit(constants.sig_kill_bad)

    def _get_inventory_id(self):
        systems = self.conn._fetch_system_by_machine_id()
        if len(systems) == 1 and 'id' in systems[0]:
            return systems[0].get('id')
        else:
            logger.error('Failed to find system in Inventory')
            exit(constants.sig_kill_bad)

    def _results_file(self, archive_dir, profile):
        return os.path.join(
            archive_dir,
            'oscap_results-{0}.xml'.format(profile['attributes']['ref_id'])
        )



"""
obdc configuration file - Files
===============================

Shared mappers for parsing and extracting data from ``/etc/odbc.ini`` and
``/etc/odbcinst.ini`` files. Parsers contained in this module are:

ODBCIni - file ``/etc/odbc.ini``
--------------------------------

ODBCinstIni - file ``/etc/odbcinst.ini``
----------------------------------------
"""
from insights.core import IniConfigFile
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs

# Since the key values in file odbc.ini is case insensitive,
# and curent filter_list is not support case insensitive,
# will add several duplicate keys here to filter out more useful data.
filter_list = [
        '[',
        'DRIVER', 'Driver', 'driver',
        'SERVER', 'Server', 'server',
        'NO_SSPS', 'No_ssps', 'no_ssps',
]
add_filter(Specs.odbc_ini, filter_list)


@parser(Specs.odbc_ini)
class ODBCIni(IniConfigFile):
    """
    The ``/etc/odbc.ini`` file is in a standard '.ini' format,
    and this parser uses the IniConfigFile base class to read this.

    Sample command output::

        [myodbc5w]
        Driver       = /usr/lib64/libmyodbc5w.so
        Description  = DSN to MySQL server
        SERVER       = localhost
        NO_SSPS     = 1

        [mysqlDSN]
        Driver      = /usr/lib64/libmyodbc5.so
        SERVER      = localhost

        [myodbc]
        Driver=MySQL
        SERVER=localhost
        #NO_SSPS=1

    Example:
        >>> odbcini.sections()
        ['myodbc5w', 'mysqlDSN', 'myodbc']
        >>> odbcini.has_option('myodbc5w', 'Driver')
        True
        >>> odbcini.get('myodbc5w', 'Driver')
        '/usr/lib64/libmyodbc5w.so'
        >>> odbcini.getint('myodbc5w', 'NO_SSPS')
        1
    """
    pass


@parser(Specs.odbcinst_ini)
class ODBCinstIni(IniConfigFile):
    """
    The ``/etc/odbcinst.ini`` file is in a standard '.ini' format,
    and this parser uses the IniConfigFile base class to read this.

    Sample command output::

        # Driver from the postgresql-odbc package
        # Setup from the unixODBC package
        [PostgreSQL]
        Description     = ODBC for PostgreSQL
        Driver          = /usr/lib/psqlodbcw.so
        Setup           = /usr/lib/libodbcpsqlS.so
        Driver64        = /usr/lib64/psqlodbcw.so
        Setup64         = /usr/lib64/libodbcpsqlS.so
        FileUsage       = 1

        # Driver from the mysql-connector-odbc package
        # Setup from the unixODBC package
        [MySQL]
        Description	= ODBC for MySQL
        Driver		= /usr/lib/libmyodbc5.so
        Setup		= /usr/lib/libodbcmyS.so
        Driver64	= /usr/lib64/libmyodbc5.so
        Setup64		= /usr/lib64/libodbcmyS.so
        FileUsage	= 1

    Example:
        >>> odbcinstinit.sections()
        ['PostgreSQL', 'MySQL']
        >>> odbcinstinit.has_option('PostgreSQL', 'Driver')
        True
        >>> odbcinstinit.get('PostgreSQL', 'Driver')
        '/usr/lib/psqlodbcw.so'
        >>> odbcinstinit.getint('PostgreSQL', 'FileUsage')
        1
    """
    pass



"""
Custom datasources for awx_manage information
"""
from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.core.filters import get_filters
from insights.specs import Specs
import json
import collections


class LocalSpecs(Specs):
    """ Local specs used only by awx_manage datasources """

    awx_manage_check_license_data_raw = simple_command("/usr/bin/awx-manage check_license --data")
    """ Returns the output of command ``/usr/bin/awx-manage check_license --data`` """


@datasource(LocalSpecs.awx_manage_check_license_data_raw, HostContext)
def awx_manage_check_license_data_datasource(broker):
    """
    This datasource provides the not-sensitive information collected
    from ``/usr/bin/awx-manage check_license --data``.

    Typical content of ``/usr/bin/awx-manage check_license --data`` file is::

        {"contact_email": "test@redhat.com", "company_name": "test Inc", "instance_count": 100, "license_date": 1655092799, "license_type": "enterprise", "subscription_name": "Red Hat Ansible Automation, Standard (100 Managed Nodes)", "sku": "MCT3691", "support_level": "Standard", "product_name": "Red Hat Ansible Automation Platform", "valid_key": true, "satellite": null, "pool_id": "2c92808179803e530179ea5989a157a4", "current_instances": 1, "available_instances": 100, "free_instances": 99, "time_remaining": 29885220, "trial": false, "grace_period_remaining": 32477220, "compliant": true, "date_warning": false, "date_expired": false}

    Returns:
        str: JSON string containing non-sensitive information.

    Raises:
        SkipComponent: When the filter/path does not exist or any exception occurs.
    """
    try:
        filters = get_filters(Specs.awx_manage_check_license_data)
        content = broker[LocalSpecs.awx_manage_check_license_data_raw].content
        if content and filters:
            json_data = json.loads(content[0])
            filter_result = {}
            for item in filters:
                filter_result[item] = json_data.get(item)
            if filter_result:
                return DatasourceProvider(content=json.dumps(collections.OrderedDict(sorted(filter_result.items()))), relative_path='insights_commands/awx-manage_check_license_--data')
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent



"""
Red Hat Release
===============

Combiner for Red Hat Release information. It uses the results of
the ``Uname`` parser and the ``RedhatRelease`` parser to determine the release
major and minor version.  ``Uname`` is the preferred source of data.
The Red Hat Release is in obtained from the system in the form major.minor.
For example, for a Red Hat Enterprise Linux 7.2 system, the release would be
major = 7, minor = 2 and rhel = '7.2'.

"""

from collections import namedtuple
from insights.core.plugins import combiner
from insights.parsers.redhat_release import RedhatRelease as rht_release
from insights.parsers.uname import Uname
from insights.core.serde import serializer, deserializer
from insights.parsers import SkipComponent
from insights.util import deprecated


Release = namedtuple("Release", field_names=["major", "minor"])
"""namedtuple: Type for storing the release information."""


@serializer(Release)
def serialize(obj, root=None):
    return {"major": obj.major, "minor": obj.minor}


@deserializer(Release)
def deserialize(_type, obj, root=None):
    return Release(**obj)


@combiner([rht_release, Uname])
def redhat_release(rh_release, un):
    """
    .. warning::
        This combiner methode is deprecated, please use
        :py:class:`insights.combiners.redhat_release.RedHatRelease` instead.

    Combiner method to check uname and redhat-release for rhel major/minor
    version.

    Prefer uname to redhat-release.

    Returns:
        Release: A named tuple with the following items:
            - major: integer
            - minor: integer

    Raises:
        SkipComponent: If the version can't be determined even though a Uname
            or RedhatRelease was provided.

    Examples:
        >>> rh_release.major
        7
        >>> rh_release.minor
        2
        >>> rh_release
        Release(major=7, minor=2)

    """

    deprecated(redhat_release, "Use the `RedHatRelease` class instead.")

    if un and un.release_tuple[0] != -1:
        return Release(*un.release_tuple)

    if rh_release:
        return Release(rh_release.major, rh_release.minor)

    raise SkipComponent("Unabled to determine release.")


@combiner([Uname, rht_release])
class RedHatRelease(object):
    """
    Combiner class to check uname and redhat-release for RHEL major/minor
    version.
    Prefer uname to redhat-release.

    Attributes:
        major (int): The major RHEL version.
        minor (int): The minor RHEL version.
        rhel (str): The RHEL version, e.g. '7.2', '7.5-0.14'
        rhel6 (str): The RHEL version when it's RHEL6, otherwise None
        rhel7 (str): The RHEL version when it's RHEL7, otherwise None
        rhel8 (str): The RHEL version when it's RHEL8, otherwise None
        rhel9 (str): The RHEL version when it's RHEL9, otherwise None

    Raises:
        SkipComponent: If the version can't be determined even though a Uname
            or RedhatRelease was provided.

    Examples:
        >>> rh_rel.rhel
        '7.2'
        >>> rh_rel.major
        7
        >>> rh_rel.minor
        2
        >>> rh_rel.rhel6 is None
        True
        >>> rh_rel.rhel7
        '7.2'
        >>> rh_rel.rhel8 is None
        True
    """
    def __init__(self, uname, rh_rel):
        self.major = self.minor = self.rhel = None
        if uname and uname.redhat_release.major != -1:
            self.major = uname.redhat_release.major
            self.minor = uname.redhat_release.minor
            self.rhel = '{0}.{1}'.format(self.major, self.minor)
        elif rh_rel and rh_rel.is_rhel:
            self.major = rh_rel.major
            self.minor = rh_rel.minor
            self.rhel = rh_rel.version

        if self.rhel is None:
            raise SkipComponent("Unable to determine release.")

        self.rhel6 = self.rhel if self.major == 6 else None
        self.rhel7 = self.rhel if self.major == 7 else None
        self.rhel8 = self.rhel if self.major == 8 else None
        self.rhel9 = self.rhel if self.major == 9 else None


@serializer(RedHatRelease)
def serialize_RedHatRelease(obj, root=None):
    return {
            "major": obj.major,
            "minor": obj.minor,
            "rhel": obj.rhel,
            "rhel6": obj.rhel6,
            "rhel7": obj.rhel7,
            "rhel8": obj.rhel8,
            "rhel9": obj.rhel9,
    }


@deserializer(RedHatRelease)
def deserialize_RedHatRelease(_type, obj, root=None):
    foo = _type.__new__(_type)
    foo.major = obj.get("major")
    foo.minor = obj.get("minor")
    foo.rhel = obj.get("rhel")
    foo.rhel6 = obj.get("rhel6")
    foo.rhel7 = obj.get("rhel7")
    foo.rhel8 = obj.get("rhel8")
    foo.rhel9 = obj.get("rhel9")
    return foo



from insights.tests import context_wrap
from insights.parsers.cobbler_settings import CobblerSettings

setting_content = """
---
# cobbler settings file
# restart cobblerd and run "cobbler sync" after making changes
# This config file is in YAML 1.0 format
# see http://yaml.org
# ==========================================================
# if 1, cobbler will allow insertions of system records that duplicate
# the --dns-name information of other system records.  In general,
# this is undesirable and should be left 0.
allow_duplicate_hostnames: 0

# if 1, cobbler will allow insertions of system records that duplicate
# the ip address information of other system records.  In general,
# this is undesirable and should be left 0.
allow_duplicate_ips: 0

# if 1, cobbler will allow insertions of system records that duplicate
# the mac address information of other system records.  In general,
# this is undesirable.
allow_duplicate_macs: 0

# by default, installs are *not* set to send installation logs to the cobbler
# # # server.  With 'anamon_enabled', kickstart templates may use the pre_anamon
# # # snippet to allow remote live monitoring of their installations from the
# # # cobbler server.  Installation logs will be stored under
# # # /var/log/cobbler/anamon/.  NOTE: This does allow an xmlrpc call to send logs
# # # to this directory, without authentication, so enable only if you are
# # # ok with this limitation.
anamon_enabled: 0

# Email out a report when cobbler finishes installing a system.
# enabled: set to 1 to turn this feature on
# sender: optional
# email: which addresses to email
# smtp_server: used to specify another server for an MTA
# subject: use the default subject unless overridden
build_reporting_enabled: 1
build_reporting_sender: "cobbler"
build_reporting_email: [ 'test@foo.example.com' ]
build_reporting_smtp_server: "192.0.2.1"
build_reporting_subject: ""

# Cheetah-language kickstart templates can import Python modules.
# while this is a useful feature, it is not safe to allow them to
# import anything they want. This whitelists which modules can be
# imported through Cheetah.  Users can expand this as needed but
# should never allow modules such as subprocess or those that
# allow access to the filesystem as Cheetah templates are evaluated
# by cobblerd as code.
cheetah_import_whitelist:
   - "random"
   - "re"
   - "time"

# Default createrepo_flags to use for new repositories. If you have
# createrepo >= 0.4.10, consider "-c cache --update -C", which can
# dramatically improve your "cobbler reposync" time.  "-s sha"
# enables working with Fedora repos from F11/F12 from EL-4 or
# EL-5 without python-hashlib installed (which is not available
# on EL-4)
createrepo_flags: "-c cache -s sha"

# if no kickstart is specified to profile add, use this template
default_kickstart: /var/lib/cobbler/kickstarts/default.ks

# configure all installed systems to use these nameservers by default
# unless defined differently in the profile.  For DHCP configurations
# you probably do /not/ want to supply this.
default_name_servers: []

# if using the authz_ownership module (see the Wiki), objects
# created without specifying an owner are assigned to this
# owner and/or group.  Can be a comma seperated list.
default_ownership:
    - "admin"

# cobbler has various sample kickstart templates stored
# in /var/lib/cobbler/kickstarts/.  This controls
# what install (root) password is set up for those
# systems that reference this variable.  The factory
# default is "cobbler" and cobbler check will warn if
# this is not changed.
default_password_crypted: "password123"

# for libvirt based installs in koan, if no virt bridge
# is specified, which bridge do we try?  For EL 4/5 hosts
# this should be xenbr0, for all versions of Fedora, try
# "virbr0".  This can be overriden on a per-profile
# basis or at the koan command line though this saves
# typing to just set it here to the most common option.
default_virt_bridge: xenbr0

# use this as the default disk size for virt guests (GB)
default_virt_file_size: 5

# use this as the default memory size for virt guests (MB)
default_virt_ram: 512

# if koan is invoked without --virt-type and no virt-type
# is set on the profile/system, what virtualization type
# should be assumed?  Values: xenpv, xenfv, qemu, vmware
# (NOTE: this does not change what virt_type is chosen by import)
default_virt_type: xenpv

# controls whether cobbler will add each new profile entry to the default
# PXE boot menu.  This can be over-ridden on a per-profile
# basis when adding/editing profiles with --enable-menu=0/1.  Users
# should ordinarily leave this setting enabled unless they are concerned
# with accidental reinstalls from users who select an entry at the PXE
# boot menu.  Adding a password to the boot menus templates
# may also be a good solution to prevent unwanted reinstallations
enable_menu: 1

# enable Func-integration?  This makes sure each installed machine is set up
# to use func out of the box, which is a powerful way to script and control
# remote machines.
# Func lives at http://fedorahosted.org/func
# read more at https://fedorahosted.org/cobbler/wiki/FuncIntegration
# you will need to mirror Fedora/EPEL packages for this feature, so see
# https://fedorahosted.org/cobbler/wiki/ManageYumRepos if you want cobbler
# to help you with this
func_auto_setup: 0
func_master: overlord.example.org

# change this port if Apache is not running plaintext on port
# 80.  Most people can leave this alone.
http_port: 80

# kernel options that should be present in every cobbler installation.
# kernel options can also be applied at the distro/profile/system
# level.
kernel_options:
    ksdevice: bootif
    lang: ' '
    text: ~

# s390 systems require additional kernel options in addition to the
# above defaults
kernel_options_s390x:
    RUNKS: 1
    ramdisk_size: 40000
    root: /dev/ram0
    ro: ~
    ip: off
    vnc: ~

# configuration options if using the authn_ldap module. See the
# the Wiki for details.  This can be ignored if you are not using
# LDAP for WebUI/XMLRPC authentication.
ldap_server: "ldap.example.com"
ldap_base_dn: "DC=example,DC=com"
ldap_port: 389
ldap_tls: 1
ldap_anonymous_bind: 1
ldap_search_bind_dn: ''
ldap_search_passwd: ''
ldap_search_prefix: 'uid='

# cobbler has a feature that allows for integration with config management
# systems such as Puppet.  The following parameters work in conjunction with
# --mgmt-classes  and are described in furhter detail at:
# https://fedorahosted.org/cobbler/wiki/UsingCobblerWithConfigManagementSystem
mgmt_classes: []
mgmt_parameters:
   from_cobbler: 1

# set to 1 to enable Cobbler's DHCP management features.
# the choice of DHCP management engine is in /etc/cobbler/modules.conf
manage_dhcp: 0

# set to 1 to enable Cobbler's DNS management features.
# the choice of DNS mangement engine is in /etc/cobbler/modules.conf
manage_dns: 0

# if using BIND (named) for DNS management in /etc/cobbler/modules.conf
# and manage_dns is enabled (above), this lists which zones are managed
# See the Wiki (https://fedorahosted.org/cobbler/wiki/ManageDns) for more info
manage_forward_zones: []
manage_reverse_zones: []

# if using cobbler with manage_dhcp, put the IP address
# of the cobbler server here so that PXE booting guests can find it
# if you do not set this correctly, this will be manifested in TFTP open timeouts.
next_server: "NotReallyThere"



# settings for power management features.  optional.
# see https://fedorahosted.org/cobbler/wiki/PowerManagement to learn more
# choices:
#    bullpap wti apc apc_snmp ether-wake ipmilan
#    drac ipmitool ilo rsa lpar bladecenter virsh
power_management_default_type: 'ipmitool'

# the commands used by the power management module are sourced
# from what directory?
power_template_dir: "/etc/cobbler/power"

# if this setting is set to 1, cobbler systems that pxe boot
# will request at the end of their installation to toggle the
# --netboot-enabled record in the cobbler system record.  This eliminates
# the potential for a PXE boot loop if the system is set to PXE
# first in it's BIOS order.  Enable this if PXE is first in your BIOS
# boot order, otherwise leave this disabled.   See the manpage
# for --netboot-enabled.
pxe_just_once: 1

# the templates used for PXE config generation are sourced
# from what directory?
pxe_template_dir: "/etc/cobbler/pxe"

# Are you using a Red Hat management platform in addition to Cobbler?
# Cobbler can help you register to it.  Choose one of the following:
#   "off"    : I'm not using Red Hat Network, Satellite, or Spacewalk
#   "hosted" : I'm using Red Hat Network
#   "site"   : I'm using Red Hat Satellite Server or Spacewalk
# You will also want to read: https://fedorahosted.org/cobbler/wiki/TipsForRhn
redhat_management_type: "site"

# if redhat_management_type is enabled, choose your server
#   "management.example.org" : For Satellite or Spacewalk
#   "xmlrpc.rhn.redhat.com"  : For Red Hat Network
# This setting is also used by the code that supports using Spacewalk/Satellite users/passwords
# within Cobbler Web and Cobbler XMLRPC.  Using RHN Hosted for this is not supported.
# This feature can be used even if redhat_management_type is off, you just have
# to have authn_spacewalk selected in modules.conf
redhat_management_server: "NotReallyThere"

# specify the default Red Hat authorization key to use to register
# system.  If left blank, no registration will be attempted.  Similarly
# you can set the --redhat-management-key to blank on any system to
# keep it from trying to register.
redhat_management_key: ""

# if using authn_spacewalk in modules.conf to let cobbler authenticate
# against Satellite/Spacewalk's auth system, by default it will not allow per user
# access into Cobbler Web and Cobbler XMLRPC.
# in order to permit this, the following setting must be enabled HOWEVER
# doing so will permit all Spacewalk/Satellite users of certain types to edit all
# of cobbler's configuration.
# these roles are:  config_admin and org_admin
# users should turn this on only if they want this behavior and
# do not have a cross-multi-org seperation concern.  If you have
# a single org in your satellite, it's probably safe to turn this
# on and then you can use CobblerWeb alongside a Satellite install.
redhat_management_permissive: 0

# if set to 1, allows /usr/bin/cobbler-register (part of the koan package)
# to be used to remotely add new cobbler system records to cobbler.
# this effectively allows for registration of new hardware from system
# records.
register_new_installs: 0

# Flags to use for yum's reposync.  If your version of yum reposync
# does not support -l, you may need to remove that option.
reposync_flags: "-l -m -d"

# when DHCP and DNS management are enabled, cobbler sync can automatically
# restart those services to apply changes.  The exception for this is
# if using ISC for DHCP, then omapi eliminates the need for a restart.
# omapi, however, is experimental and not recommended for most configurations.
# If DHCP and DNS are going to be managed, but hosted on a box that
# is not on this server, disable restarts here and write some other
# script to ensure that the config files get copied/rsynced to the destination
# box.  This can be done by modifying the restart services trigger.
# Note that if manage_dhcp and manage_dns are disabled, the respective
# parameter will have no effect.  Most users should not need to change
# this.
restart_dns: 1
restart_dhcp: 1

# install triggers are scripts in /var/lib/cobbler/triggers/install
# that are triggered in kickstart pre and post sections.  Any
# executable script in those directories is run.  They can be used
# to send email or perform other actions.  They are currently
# run as root so if you do not need this functionality you can
# disable it, though this will also disable "cobbler status" which
# uses a logging trigger to audit install progress.
run_install_triggers: 1

# enables a trigger which version controls all changes to /var/lib/cobbler
# when add, edit, or sync events are performed.  This can be used
# to revert to previous database versions, generate RSS feeds, or for
# other auditing or backup purposes.  git is the recommend SCM
# for use with this feature.
scm_track_enabled: 0
scm_track_mode: "git"

# this is the address of the cobbler server -- as it is used
# by systems during the install process, it must be the address
# or hostname of the system as those systems can see the server.
# if you have a server that appears differently to different subnets
# (dual homed, etc), you need to read the --server-override section
# of the manpage for how that works.
server: NotReallyThere


# this is a directory of files that cobbler uses to make
# templating easier.  See the Wiki for more information.  Changing
# this directory should not be required.
snippetsdir: /var/lib/cobbler/snippets

# Normally if a kickstart is specified at a remote location, this
# URL will be passed directly to the kickstarting system, thus bypassing
# the usual snippet templating Cobbler does for local kickstart files. If
# this option is enabled, Cobbler will fetch the file contents internally
# and serve a templated version of the file to the client.
template_remote_kickstarts: 0

# should new profiles for virtual machines default to auto booting with the physical host when the physical host reboots?
# this can be overridden on each profile or system object.
virt_auto_boot: 1

# cobbler's web directory.  Don't change this setting -- see the
# Wiki on "relocating your cobbler install" if your /var partition
# is not large enough.
webdir: /var/www/cobbler

# cobbler's public XMLRPC listens on this port.  Change this only
# if absolutely needed, as you'll have to start supplying a new
# port option to koan if it is not the default.
xmlrpc_port: 25151

# "cobbler repo add" commands set cobbler up with repository
# information that can be used during kickstart and is automatically
# set up in the cobbler kickstart templates.  By default, these
# are only available at install time.  To make these repositories
# usable on installed systems (since cobbler makes a very convient)
# mirror, set this to 1.  Most users can safely set this to 1.  Users
# who have a dual homed cobbler server, or are installing laptops that
# will not always have access to the cobbler server may wish to leave
# this as 0.  In that case, the cobbler mirrored yum repos are still
# accessable at http://cobbler.example.org/cblr/repo_mirror and yum
# configuration can still be done manually.  This is just a shortcut.
yum_post_install_mirror: 1

# the default yum priority for all the distros.  This is only used
# if yum-priorities plugin is used.  1=maximum.  Tweak with caution.
yum_distro_priority: 1

# Flags to use for yumdownloader.  Not all versions may support
# --resolve.
yumdownloader_flags: "--resolve"

# Safe Tempating disables several features of cheetah that may prove
# to be undesirable in situations where cobbler_web is used.  We
#  recommend that you enable safe templating any time you have
# individuals writting templates that do not have root access to the
# cobbler system.
safe_templating: true
""".strip()


def test_cobbler_setting():
    result = CobblerSettings(context_wrap(setting_content))
    assert result.get("safe_templating") is True
    assert result.get("mgmt_parameters") == {'from_cobbler': 1}



import doctest
import pytest

from insights.parsers import ls_var_www_perms
from insights.parsers.ls_var_www_perms import LsVarWwwPerms
from insights.tests import context_wrap
from insights.util.file_permissions import FilePermissions


GOOD_FOLDER_OUTPUT = """
crw-rw-rw-. 1 root root 1, 3 Dec 18 09:18 /dev/null

/var/www:
total 16
drwxr-xr-x.  4 root root  33 Dec 15 08:12 .
drwxr-xr-x. 20 root root 278 Dec 15 08:12 ..
drwxr-xr-x.  2 root root   6 Oct  3 09:37 cgi-bin
drwxr-xr-x.  2 root root   6 Oct  3 09:37 html
"""

GOOD_FOLDER_EXPECTED_OUTPUT = [
    "drwxr-xr-x.  4 root root  33 Dec 15 08:12 .",
    "drwxr-xr-x. 20 root root 278 Dec 15 08:12 ..",
    "drwxr-xr-x.  2 root root   6 Oct  3 09:37 cgi-bin",
    "drwxr-xr-x.  2 root root   6 Oct  3 09:37 html",
]

EMPTY_FOLDER_OUTPUT = """
crw-rw-rw-. 1 root root 1, 3 Dec 18 09:18 /dev/null

/var/www:
total 0
drwxr-xr-x.  4 root root  33 Dec 15 08:12 .
drwxr-xr-x. 20 root root 278 Dec 15 08:12 ..
"""

EMPTY_FOLDER_EXPECTED_OUTPUT = [
    "drwxr-xr-x.  4 root root  33 Dec 15 08:12 .",
    "drwxr-xr-x. 20 root root 278 Dec 15 08:12 ..",
]

NO_FOLDER_OUTPUT = """
/bin/ls: cannot access '/var/www': No such file or directory
crw-rw-rw-. 1 root root 1, 3 Dec 18 09:18 /dev/null
"""

NO_FOLDER_EXPECTED_OUTPUT = [
]

OUTPUTS_FROM_FOLDERS = [
    (GOOD_FOLDER_OUTPUT, GOOD_FOLDER_EXPECTED_OUTPUT),
    (EMPTY_FOLDER_OUTPUT, EMPTY_FOLDER_EXPECTED_OUTPUT),
    (NO_FOLDER_OUTPUT, NO_FOLDER_EXPECTED_OUTPUT)
]


@pytest.mark.parametrize("output, expected_output", OUTPUTS_FROM_FOLDERS)
def test_ls_var(output, expected_output):
    test = LsVarWwwPerms(context_wrap(output))
    assert len(test.file_permissions) == len(expected_output)
    for test_line, expected_line in zip(test.file_permissions, expected_output):
        assert repr(test_line) == repr(FilePermissions(expected_line))
        assert test_line.line == FilePermissions(expected_line).line


def test_doc_examples():
    env = {
        "ls_var_www_perms": LsVarWwwPerms(context_wrap(GOOD_FOLDER_OUTPUT))
    }
    failed, total = doctest.testmod(ls_var_www_perms, globs=env)
    assert failed == 0



"""
Custom datasource to get ssl certificate file path.
"""

from insights.combiners.httpd_conf import HttpdConfTree
from insights.combiners.nginx_conf import NginxConfTree
from insights.parsers.mssql_conf import MsSQLConf
from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource


@datasource(HttpdConfTree, HostContext)
def httpd_certificate_info_in_nss(broker):
    """
    Get the certificate info configured in nss database

    Arguments:
        broker: the broker object for the current session

    Returns:
        list: Returns a list of tuple with the Nss database path and the certificate nickname

    Raises:
        SkipComponent: Raised when NSSEngine isn't enabled or "NSSCertificateDatabase" and "NSSNickname" directives aren't found
    """
    conf = broker[HttpdConfTree]
    path_pairs = []
    virtual_hosts = conf.find('VirtualHost')
    for host in virtual_hosts:
        nss_engine = nss_database = cert_name = None
        nss_engine = host.select('NSSEngine')
        nss_database = host.select('NSSCertificateDatabase')
        cert_name = host.select('NSSNickname')
        if nss_engine and nss_engine.value and nss_database and cert_name:
            path_pairs.append((nss_database[0].value, cert_name[0].value))
    if path_pairs:
        return path_pairs
    raise SkipComponent


@datasource(HttpdConfTree, HostContext)
def httpd_ssl_certificate_files(broker):
    """
    Get the httpd SSL certificate file path configured by "SSLCertificateFile"

    Arguments:
        broker: the broker object for the current session

    Returns:
        str: Returns the SSL certificate file path configured by "SSLCertificateFile"

    Raises:
        SkipComponent: Raised if "SSLCertificateFile" directive isn't found
    """
    conf = broker[HttpdConfTree]
    virtual_hosts = conf.find('VirtualHost')
    ssl_certs = []
    for host in virtual_hosts:
        ssl_cert = ssl_engine = None
        ssl_engine = host.select('SSLEngine')
        ssl_cert = host.select('SSLCertificateFile')
        if ssl_engine and ssl_engine.value and ssl_cert:
            ssl_certs.append(str(ssl_cert.value))
    if ssl_certs:
        return ssl_certs
    raise SkipComponent


@datasource(NginxConfTree, HostContext)
def nginx_ssl_certificate_files(broker):
    """
    Get the nginx SSL certificate file path configured by "ssl_certificate"

    Arguments:
        broker: the broker object for the current session

    Returns:
        str: Returns the SSL certificate file path configured by "ssl_certificate"

    Raises:
        SkipComponent: Raised if "ssl_certificate" directive isn't found
    """
    conf = broker[NginxConfTree]
    ssl_certs = conf.find('ssl_certificate')
    if ssl_certs:
        return [str(ssl_cert.value) for ssl_cert in ssl_certs]
    raise SkipComponent


@datasource(MsSQLConf, HostContext)
def mssql_tls_cert_file(broker):
    """
    Get the mssql tls certificate file path configured by "ssl_certificate"
    Arguments:
        broker: the broker object for the current session
    Returns:
        str: Returns the SSL certificate file path configured by "ssl_certificate"
    Raises:
        SkipComponent: Raised if "ssl_certificate" directive isn't found
    """
    mssql_conf_content = broker[MsSQLConf]
    if mssql_conf_content.has_option("network", "tlscert"):
        return mssql_conf_content.get("network", "tlscert")
    raise SkipComponent



"""
ImageMagickPolicy - files ``/etc/ImageMagick/policy.xml`` and ``/usr/lib*/ImageMagick-6.5.4/config/policy.xml``
===============================================================================================================
"""

from insights.core import XMLParser
from insights.core.plugins import parser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.imagemagick_policy)
class ImageMagickPolicy(XMLParser):
    """
    Class for parsing the ``/etc/ImageMagick/policy.xml`` and ``/usr/lib*/ImageMagick-6.5.4/config/policy.xml``
    files.

    Attributes:
        policies (list): list of Element objects with a 'policy' tag

    Raises:
        SkipException: When content is empty or cannot be parsed.

    Sample output of this file is::

        <?xml version="1.0" encoding="UTF-8"?>
        <!DOCTYPE policymap [
          <!ELEMENT policymap (policy)+>
          <!ATTLIST policymap xmlns CDATA #FIXED ''>
          <!ELEMENT policy EMPTY>
          <!ATTLIST policy xmlns CDATA #FIXED '' domain NMTOKEN #REQUIRED
            name NMTOKEN #IMPLIED pattern CDATA #IMPLIED rights NMTOKEN #IMPLIED
            stealth NMTOKEN #IMPLIED value CDATA #IMPLIED>
        ]>
        <!--
          Configure ImageMagick policies.
          ...
        -->
        <policymap>
          <!-- <policy domain="system" name="shred" value="2"/> -->
          <!-- <policy domain="system" name="precision" value="6"/> -->
          <!-- <policy domain="system" name="memory-map" value="anonymous"/> -->
          <policy domain="coder" rights="none" pattern="EPHEMERAL"/>
          <policy domain="coder" rights="none" pattern="HTTPS"/>
          <policy domain="coder" rights="none" pattern="HTTP"/>
          <policy domain="coder" rights="none" pattern="URL"/>
          <policy domain="coder" rights="none" pattern="FTP"/>
          <policy domain="coder" rights="none" pattern="MVG"/>
          <policy domain="coder" rights="none" pattern="MSL"/>
          <policy domain="coder" rights="none" pattern="TEXT" />
          <policy domain="coder" rights="none" pattern="LABEL"/>
          <policy domain="path" rights="none" pattern="@*"/>
        </policymap>

    Examples:
        >>> type(imagemagick_policy)
        <class 'insights.parsers.imagemagick_policy.ImageMagickPolicy'>
        >>> len(imagemagick_policy.policies)
        10
        >>> sorted(imagemagick_policy.policies[0].items())
        [('domain', 'coder'), ('pattern', 'EPHEMERAL'), ('rights', 'none')]
    """

    def parse_content(self, content):
        if not content:
            raise SkipException("No content.")

        try:
            super(ImageMagickPolicy, self).parse_content(content)
            self.policies = self.get_elements(".//policy")
        except Exception:  # file without elements
            self.policies = []



import pytest
import doctest
from insights.parsers import open_vm_tools, SkipException
from insights.parsers.open_vm_tools import OpenVmToolsStatRawTextSession
from insights.tests import context_wrap

V_OUT1 = """
vmware-toolbox-cmd must be run inside a virtual machine.
""".strip()

V_OUT2 = """
test
""".strip()

V_OUT3 = """
session = 4004861987670969122
uptime = 1036293956
version = VMware ESXi 6.0.0 build-12345
provider =
uuid.bios = 00 00 00 00 00 00 66 8e-00 00 00 00 51 1e 23 f3
""".strip()


def test_OpenVmToolsStatRawTextSession():
    with pytest.raises(SkipException):
        OpenVmToolsStatRawTextSession(context_wrap(V_OUT1))

    with pytest.raises(SkipException):
        OpenVmToolsStatRawTextSession(context_wrap(V_OUT2))

    o1 = OpenVmToolsStatRawTextSession(context_wrap(V_OUT3))
    assert o1['version'] == 'VMware ESXi 6.0.0 build-12345'
    assert o1['provider'] == ''


def test_doc_examples():
    env = {
            'ovmt': OpenVmToolsStatRawTextSession(context_wrap(V_OUT3)),
          }
    failed, total = doctest.testmod(open_vm_tools, globs=env)
    assert failed == 0



from insights.parsers.engine_config import EngineConfigAll
from insights.tests import context_wrap


ENGINE_CONFIG_ALL_OUTPUT = """
AffinityRulesEnforcementManagerEnabled: true version: general
AffinityRulesEnforcementManagerRegularInterval: 1 version: general
AsyncTaskPollingRate: 10 version: general
AsyncTaskZombieTaskLifeInMinutes: 3000 version: general
AuditLogAgingThreshold: 30 version: general
AuditLogCleanupTime: 03:35:35 version: general
BlockMigrationOnSwapUsagePercentage: 0 version: general
BootstrapMinimalVdsmVersion: 4.9 version: general
CpuOverCommitDurationMinutes: 2 version: general
DisableFenceAtStartupInSec: 300 version: general
PopulateDirectLUNDiskDescriptionWithLUNId: 4 version: general
MaxBlockDiskSize: 8192 version: general
ClusterEmulatedMachines: pc-i440fx-rhel7.2.0,pseries-rhel7.2.0 version: 3.6
ClusterEmulatedMachines: pc-i440fx-rhel7.2.0,pseries-rhel7.2.0 version: 4.0
ClusterEmulatedMachines: pc-i440fx-rhel7.3.0,pc-i440fx-2.6,pseries-rhel7.3.0 version: 4.1
ClusterEmulatedMachines: rhel6.2.0 version: 3.0
ClusterEmulatedMachines: rhel6.3.0 version: 3.1
ClusterEmulatedMachines: rhel6.4.0 version: 3.2
ClusterEmulatedMachines: rhel6.5.0 version: 3.3
ClusterEmulatedMachines: rhel6.5.0,pseries version: 3.4
ClusterEmulatedMachines: rhel6.5.0,pseries version: 3.5
WANDisableEffects: animation version: general
WANColorDepth: 16 version: general
EnableSpiceRootCertificateValidation: true version: general
EnableUSBAsDefault: true version: general
EnableVdsLoadBalancing: true version: general
EncryptHostCommunication: true version: general
ExternalCommunicationProtocol: TLSv1.2 version: general
CriticalSpaceActionBlocker: 5 version: general
WarningLowSpaceIndicator: 10 version: general
HighUtilizationForEvenlyDistribute: 75 version: general
HighUtilizationForPowerSave: 75 version: general
LogPhysicalMemoryThresholdInMB: 1024 version: general
LowUtilizationForEvenlyDistribute: 0 version: general
LowUtilizationForPowerSave: 20 version: general
MaxNumberOfHostsInStoragePool: 250 version: general
MaxNumOfCpuPerSocket: 16 version: 3.6
MaxNumOfCpuPerSocket: 16 version: 4.0
MaxNumOfCpuPerSocket: 254 version: 4.1
MaxNumOfThreadsPerCpu: 8 version: 3.6
MaxNumOfThreadsPerCpu: 8 version: 4.0
MaxNumOfThreadsPerCpu: 8 version: 4.1
MaxNumOfVmCpus: 240 version: 3.6
MaxNumOfVmCpus: 240 version: 4.0
MaxNumOfVmCpus: 288 version: 4.1
MaxNumOfVmSockets: 16 version: 3.6
MaxNumOfVmSockets: 16 version: 4.0
AffMaxNumOfVmSockets: 16 version: 4.1
MaxRerunVmOnVdsCount: 3 version: general
MaxSchedulerWeight: 1000 version: general
MaxStorageVdsDelayCheckSec: 5 version: general
MaxStorageVdsTimeoutCheckSec: 30 version: general
MaxVdsMemOverCommit: 200 version: general
MaxVdsMemOverCommitForServers: 150 version: general
MaxVdsNameLength: 255 version: general
MaxVmNameLength: 64 version: general
MaxVmNameLengthSysprep: 15 version: general
MaxVmsInPool: 1000 version: general
VmPoolMaxSubsequentFailures: 3 version: general
NumberOfFailedRunsOnVds: 3 version: general
NumberVmRefreshesBeforeSave: 5 version: general
oVirtISOsRepositoryPath: /usr/share/ovirt-node-iso:/usr/share/rhev-hypervisor version: general
OvfItemsCountPerUpdate: 100 version: general
OvfUpdateIntervalInMinutes: 60 version: general
StorageDomainOvfStoreCount: 2 version: general
ProductRPMVersion: 4.1.8.2-0.1.el7 version: general
SANWipeAfterDelete: false version: general
SearchResultsLimit: 100 version: general
ServerRebootTimeout: 300 version: general
ConsoleReleaseCursorKeys: shift+f12 version: general
ConsoleToggleFullScreenKeys: shift+f11 version: general
SpiceUsbAutoShare: true version: general
FullScreenWebadminDefault: false version: general
FullScreenUserportalBasicDefault: true version: general
FullScreenUserportalExtendedDefault: false version: general
SpmCommandFailOverRetries: 3 version: general
SPMFailOverAttempts: 3 version: general
SpmVCpuConsumption: 1 version: general
SSHInactivityTimeoutSeconds: 300 version: general
SSHInactivityHardTimeoutSeconds: 1800 version: general
NumberOfUSBSlots: 4 version: general
SSLEnabled: true version: general
StorageDomainFailureTimeoutInMinutes: 5 version: general
StoragePoolRefreshTimeInSeconds: 10 version: general
TimeoutToResetVdsInSeconds: 60 version: general
DelayResetForSpmInSeconds: 20 version: general
DelayResetPerVmInSeconds: 0.5 version: general
TimeToReduceFailedRunOnVdsInMinutes: 30 version: general
UserDefinedVMProperties:  version: 3.6
UserDefinedVMProperties:  version: 4.0
UserDefinedVMProperties:  version: 4.1
UtilizationThresholdInPercent: 80 version: general
ValidNumOfMonitors: 1,2,4 version: general
VdcVersion: 4.1.0.0 version: general
VDSAttemptsToResetCount: 2 version: general
VdsLoadBalancingIntervalInMinutes: 1 version: general
VdsRecoveryTimeoutInMinutes: 3 version: general
VdsRefreshRate: 3 version: general
vdsTimeout: 180 version: general
vdsConnectionTimeout: 20 version: general
vdsRetries: 0 version: general
EventProcessingPoolSize: 10 version: general
VmGracefulShutdownMessage: System Administrator has initiated shutdown of this Virtual Machine. Virtual Machine is shutting down. version: general
VmGracefulShutdownTimeout: 30 version: general
VM32BitMaxMemorySizeInMB: 20480 version: general
VM64BitMaxMemorySizeInMB: 4194304 version: 3.6
VM64BitMaxMemorySizeInMB: 4194304 version: 4.0
VM64BitMaxMemorySizeInMB: 4194304 version: 4.1
MaxMemorySlots: 16 version: general
HotPlugMemoryMultiplicationSizeMb: 256 version: general
VncKeyboardLayout: en-us version: general
WaitForVdsInitInSec: 60 version: general
FenceQuietTimeBetweenOperationsInSec: 180 version: general
FenceProxyDefaultPreferences: cluster,dc version: general
MaxAuditLogMessageLength: 10000 version: general
SysPrepDefaultUser:  version: general
SysPrepDefaultPassword: Empty version: general
UserSessionTimeOutInterval: 30 version: general
IPTablesConfig:
# oVirt default firewall configuration. Automatically generated by vdsm bootstrap script.
*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
-A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
-A INPUT -p icmp -j ACCEPT
-A INPUT -i lo -j ACCEPT
# vdsm
-A INPUT -p tcp --dport @VDSM_PORT@ -j ACCEPT
# ovirt-imageio-daemon
-A INPUT -p tcp --dport 54322 -j ACCEPT
# rpc.statd
-A INPUT -p tcp --dport 111 -j ACCEPT
-A INPUT -p udp --dport 111 -j ACCEPT
# SSH
-A INPUT -p tcp --dport @SSH_PORT@ -j ACCEPT
# snmp
-A INPUT -p udp --dport 161 -j ACCEPT
# Cockpit
-A INPUT -p tcp --dport 9090 -j ACCEPT

@CUSTOM_RULES@

# Reject any other input traffic
-A INPUT -j REJECT --reject-with icmp-host-prohibited
-A FORWARD -m physdev ! --physdev-is-bridged -j REJECT --reject-with icmp-host-prohibited
COMMIT
 version: general
IPTablesConfigSiteCustom:  version: general
OvirtIsoPrefix: ^ovirt-node-iso-([0-9].*)\.iso$:^rhevh-([0-9].*)\.iso$ version: general
OvirtInitialSupportedIsoVersion: 2.5.5:5.8 version: general
VdsLocalDisksLowFreeSpace: 500 version: general
VdsLocalDisksCriticallyLowFreeSpace: 500 version: general
JobCleanupRateInMinutes: 10 version: general
SucceededJobCleanupTimeInMinutes: 10 version: general
FailedJobCleanupTimeInMinutes: 60 version: general
VmPoolMonitorIntervalInMinutes: 5 version: general
VmPoolMonitorBatchSize: 5 version: general
NetworkConnectivityCheckTimeoutInSeconds: 120 version: general
AllowClusterWithVirtGlusterEnabled: true version: general
EnableMACAntiSpoofingFilterRules: true version: general
EnableHostTimeDrift: true version: general
EngineMode: Active version: general
HostTimeDriftInSec: 300 version: general
LogMaxPhysicalMemoryUsedThresholdInPercentage: 95 version: general
LogMaxCpuUsedThresholdInPercentage: 95 version: general
LogMaxNetworkUsedThresholdInPercentage: 95 version: general
LogMinFreeSwapThresholdInMB: 256 version: general
LogMaxSwapUsedThresholdInPercentage: 95 version: general
SpiceProxyDefault:  version: general
RemoteViewerSupportedVersions: rhev-win64:2.0-160;rhev-win32:2.0-160;rhel7:2.0-6;rhel6:99.0-1 version: general
RemoteViewerNewerVersionUrl: ${console_client_resources_url} version: general
RemapCtrlAltDelDefault: true version: general
ClientModeSpiceDefault: Native version: general
ClientModeVncDefault: Native version: general
ClientModeRdpDefault: Auto version: general
WebSocketProxy: rhevm.example.com:6100 version: general
WebSocketProxyTicketValiditySeconds: 120 version: general
CustomDeviceProperties: {type=interface;prop={SecurityGroups=^(?:(?:[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}, *)*[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}|)$}} version: 3.6
CustomDeviceProperties: {type=interface;prop={SecurityGroups=^(?:(?:[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}, *)*[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}|)$}} version: 4.0
CustomDeviceProperties: {type=interface;prop={SecurityGroups=^(?:(?:[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}, *)*[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}|)$}} version: 4.1
UserDefinedNetworkCustomProperties:  version: 3.6
UserDefinedNetworkCustomProperties:  version: 4.0
UserDefinedNetworkCustomProperties:  version: 4.1
GlusterRefreshRateHooks: 7200 version: general
DefaultWindowsTimeZone: GMT Standard Time version: general
DefaultGeneralTimeZone: Etc/GMT version: general
OnlyRequiredNetworksMandatoryForVdsSelection: false version: general
MaxAverageNetworkQoSValue: 1024 version: general
MaxPeakNetworkQoSValue: 2048 version: general
MaxBurstNetworkQoSValue: 10240 version: general
MaxHostNetworkQosShares: 100 version: general
DisplayUncaughtUIExceptions: true version: general
QoSInboundAverageDefaultValue: 10 version: general
QoSInboundPeakDefaultValue: 10 version: general
QoSInboundBurstDefaultValue: 100 version: general
QoSOutboundAverageDefaultValue: 10 version: general
QoSOutboundPeakDefaultValue: 10 version: general
QoSOutboundBurstDefaultValue: 100 version: general
SecureConnectionWithOATServers: true version: general
PollUri: AttestationService/resources/PollHosts version: general
AttestationTruststore: TrustStore.jks version: general
AttestationPort: 8443 version: general
AttestationTruststorePass:  version: general
AttestationServer:  version: general
AttestationFirstStageSize: 10 version: general
ExternalSchedulerServiceURL: http://localhost:18781/ version: general
ExternalSchedulerConnectionTimeout: 100 version: general
ExternalSchedulerEnabled: false version: general
ExternalSchedulerResponseTimeout: 120000 version: general
DwhHeartBeatInterval: 15 version: general
HostPreparingForMaintenanceIdleTime: 300 version: general
UseFqdnForRdpIfAvailable: true version: general
SpeedOptimizationSchedulingThreshold: 10 version: general
SchedulerAllowOverBooking: false version: general
SchedulerOverBookingThreshold: 10 version: general
OverUtilizationForHaReservation: 200 version: general
ScaleDownForHaReservation: 1 version: general
VdsHaReservationIntervalInMinutes: 5 version: general
DefaultMaximumMigrationDowntime: 0 version: general
DefaultSerialNumberPolicy: HOST_ID version: general
MigrationPolicies: [{"id":{"uuid":"80554327-0569-496b-bdeb-fcbbf52b827b"},"maxMigrations":2,"autoConvergence":true,"migrationCompression":false,"enableGuestEvents":true,"name":"Minimal downtime","description":"A policy that lets the VM migrate in typical situations. The VM should not experience any significant downtime. If the VM migration is not converging for a long time, the migration will be aborted. The guest agent hook mechanism is enabled.","config":{"convergenceItems":[{"stallingLimit":1,"convergenceItem":{"action":"setDowntime","params":["150"]}},{"stallingLimit":2,"convergenceItem":{"action":"setDowntime","params":["200"]}},{"stallingLimit":3,"convergenceItem":{"action":"setDowntime","params":["300"]}},{"stallingLimit":4,"convergenceItem":{"action":"setDowntime","params":["400"]}},{"stallingLimit":6,"convergenceItem":{"action":"setDowntime","params":["500"]}}],"initialItems":[{"action":"setDowntime","params":["100"]}],"lastItems":[{"action":"abort","params":[]}]}},{"id":{"uuid":"80554327-0569-496b-bdeb-fcbbf52b827c"},"maxMigrations":1,"autoConvergence":true,"migrationCompression":true,"enableGuestEvents":true,"name":"Suspend workload if needed","description":"A policy that lets the VM migrate in most situations, including VMs running heavy workloads. On the other hand, the VM may experience a more significant downtime. The migration may still be aborted for extreme workloads. The guest agent hook mechanism is enabled.","config":{"convergenceItems":[{"stallingLimit":1,"convergenceItem":{"action":"setDowntime","params":["150"]}},{"stallingLimit":2,"convergenceItem":{"action":"setDowntime","params":["200"]}},{"stallingLimit":3,"convergenceItem":{"action":"setDowntime","params":["300"]}},{"stallingLimit":4,"convergenceItem":{"action":"setDowntime","params":["400"]}},{"stallingLimit":6,"convergenceItem":{"action":"setDowntime","params":["500"]}}],"initialItems":[{"action":"setDowntime","params":["100"]}],"lastItems":[{"action":"setDowntime","params":["5000"]},{"action":"abort","params":[]}]}}] version: 3.6
MigrationPolicies: [{"id":{"uuid":"80554327-0569-496b-bdeb-fcbbf52b827b"},"maxMigrations":2,"autoConvergence":true,"migrationCompression":false,"enableGuestEvents":true,"name":"Minimal downtime","description":"A policy that lets the VM migrate in typical situations. The VM should not experience any significant downtime. If the VM migration is not converging for a long time, the migration will be aborted. The guest agent hook mechanism is enabled.","config":{"convergenceItems":[{"stallingLimit":1,"convergenceItem":{"action":"setDowntime","params":["150"]}},{"stallingLimit":2,"convergenceItem":{"action":"setDowntime","params":["200"]}},{"stallingLimit":3,"convergenceItem":{"action":"setDowntime","params":["300"]}},{"stallingLimit":4,"convergenceItem":{"action":"setDowntime","params":["400"]}},{"stallingLimit":6,"convergenceItem":{"action":"setDowntime","params":["500"]}}],"initialItems":[{"action":"setDowntime","params":["100"]}],"lastItems":[{"action":"abort","params":[]}]}},{"id":{"uuid":"80554327-0569-496b-bdeb-fcbbf52b827c"},"maxMigrations":1,"autoConvergence":true,"migrationCompression":true,"enableGuestEvents":true,"name":"Suspend workload if needed","description":"A policy that lets the VM migrate in most situations, including VMs running heavy workloads. On the other hand, the VM may experience a more significant downtime. The migration may still be aborted for extreme workloads. The guest agent hook mechanism is enabled.","config":{"convergenceItems":[{"stallingLimit":1,"convergenceItem":{"action":"setDowntime","params":["150"]}},{"stallingLimit":2,"convergenceItem":{"action":"setDowntime","params":["200"]}},{"stallingLimit":3,"convergenceItem":{"action":"setDowntime","params":["300"]}},{"stallingLimit":4,"convergenceItem":{"action":"setDowntime","params":["400"]}},{"stallingLimit":6,"convergenceItem":{"action":"setDowntime","params":["500"]}}],"initialItems":[{"action":"setDowntime","params":["100"]}],"lastItems":[{"action":"setDowntime","params":["5000"]},{"action":"abort","params":[]}]}}] version: 4.0
MigrationPolicies: [{"id":{"uuid":"80554327-0569-496b-bdeb-fcbbf52b827b"},"maxMigrations":2,"autoConvergence":true,"migrationCompression":false,"enableGuestEvents":true,"name":"Minimal downtime","description":"A policy that lets the VM migrate in typical situations. The VM should not experience any significant downtime. If the VM migration is not converging for a long time, the migration will be aborted. The guest agent hook mechanism is enabled.","config":{"convergenceItems":[{"stallingLimit":1,"convergenceItem":{"action":"setDowntime","params":["150"]}},{"stallingLimit":2,"convergenceItem":{"action":"setDowntime","params":["200"]}},{"stallingLimit":3,"convergenceItem":{"action":"setDowntime","params":["300"]}},{"stallingLimit":4,"convergenceItem":{"action":"setDowntime","params":["400"]}},{"stallingLimit":6,"convergenceItem":{"action":"setDowntime","params":["500"]}}],"initialItems":[{"action":"setDowntime","params":["100"]}],"lastItems":[{"action":"abort","params":[]}]}},{"id":{"uuid":"80554327-0569-496b-bdeb-fcbbf52b827c"},"maxMigrations":1,"autoConvergence":true,"migrationCompression":true,"enableGuestEvents":true,"name":"Suspend workload if needed","description":"A policy that lets the VM migrate in most situations, including VMs running heavy workloads. On the other hand, the VM may experience a more significant downtime. The migration may still be aborted for extreme workloads. The guest agent hook mechanism is enabled.","config":{"convergenceItems":[{"stallingLimit":1,"convergenceItem":{"action":"setDowntime","params":["150"]}},{"stallingLimit":2,"convergenceItem":{"action":"setDowntime","params":["200"]}},{"stallingLimit":3,"convergenceItem":{"action":"setDowntime","params":["300"]}},{"stallingLimit":4,"convergenceItem":{"action":"setDowntime","params":["400"]}},{"stallingLimit":6,"convergenceItem":{"action":"setDowntime","params":["500"]}}],"initialItems":[{"action":"setDowntime","params":["100"]}],"lastItems":[{"action":"setDowntime","params":["5000"]},{"action":"abort","params":[]}]}},{"id":{"uuid":"a7aeedb2-8d66-4e51-bb22-32595027ce71"},"maxMigrations":2,"autoConvergence":true,"migrationCompression":false,"enableGuestEvents":true,"name":"Post-copy migration","description":"The VM should not experience any significant downtime. If the VM migration is not converging for a long time, the migration will be switched to post-copy. The guest agent hook mechanism is enabled.","config":{"convergenceItems":[{"stallingLimit":1,"convergenceItem":{"action":"setDowntime","params":["150"]}},{"stallingLimit":2,"convergenceItem":{"action":"setDowntime","params":["200"]}}],"initialItems":[{"action":"setDowntime","params":["100"]}],"lastItems":[{"action":"postcopy","params":[]},{"action":"abort","params":[]}]}}] version: 4.1
DefaultCustomSerialNumber: Dummy serial number. version: general
HotPlugCpuSupported: {"x86":"false","ppc":"false"} version: 3.6
HotPlugCpuSupported: {"x86":"true","ppc":"false"} version: 4.0
HotPlugCpuSupported: {"x86":"true","ppc":"true"} version: 4.1
HotUnplugCpuSupported: {"x86":"false","ppc":"false"} version: 3.6
HotUnplugCpuSupported: {"x86":"false","ppc":"false"} version: 4.0
HotUnplugCpuSupported: {"x86":"true","ppc":"true"} version: 4.1
MaxNumOfTriesToRunFailedAutoStartVm: 10 version: general
RetryToRunAutoStartVmIntervalInSeconds: 30 version: general
DelayToRunAutoStartVmIntervalInSeconds: 10 version: general
MaxNumOfSkipsBeforeAutoStartVm: 3 version: general
CSRFProtection: false version: general
CORSSupport: false version: general
CORSAllowedOrigins:  version: general
PMHealthCheckEnabled: false version: general
PMHealthCheckIntervalInSec: 3600 version: general
FenceKdumpDestinationAddress:  version: general
FenceKdumpDestinationPort: 7410 version: general
FenceKdumpMessageInterval: 5 version: general
FenceKdumpListenerTimeout: 90 version: general
KdumpStartedTimeout: 30 version: general
CustomFenceAgentMapping:  version: general
CustomFenceAgentDefaultParams:  version: general
CustomFenceAgentDefaultParamsForPPC:  version: general
CustomVdsFenceOptionMapping:  version: general
CustomVdsFenceType:  version: general
CustomFencePowerWaitParam:  version: general
ClusterRequiredRngSourcesDefault:  version: 3.6
ClusterRequiredRngSourcesDefault:  version: 4.0
ClusterRequiredRngSourcesDefault:  version: 4.1
DefaultMTU: 1500 version: general
FenceStartStatusRetries: 18 version: general
FenceStartStatusDelayBetweenRetriesInSec: 10 version: general
FenceStopStatusRetries: 18 version: general
FenceStopStatusDelayBetweenRetriesInSec: 10 version: general
DefaultAutoConvergence: false version: general
DefaultMigrationCompression: false version: general
GlusterRefreshRateGeoRepStatusInSecs: 300 version: general
GlusterRefreshRateGeoRepDiscoveryInSecs: 3600 version: general
LibgfApiSupported: false version: 3.6
LibgfApiSupported: false version: 4.0
LibgfApiSupported: false version: 4.1
BackupCheckPeriodInHours: 6 version: general
BackupAlertPeriodInDays: 1 version: general
HostPackagesUpdateTimeInHours: 24 version: general
UserPackageNamesForCheckUpdate:  version: general
CertificationValidityCheckTimeInHours: 24 version: general
HostedEngineVmName: HostedEngine version: general
AutoImportHostedEngine: true version: general
AllowEditingHostedEngine: true version: general
HostedEngineConfigDiskSizeInBytes: 20480 version: general
AlertOnNumberOfLVs: 1000 version: general
HystrixMonitoringEnabled: false version: general
ImageProxyAddress: rhevm.example.com:54323 version: general
""".strip()


def test_engine_config_all_output():
    # Create output instance
    output = EngineConfigAll(context_wrap(ENGINE_CONFIG_ALL_OUTPUT))

    # Keyword with single value and version
    assert 'MaxStorageVdsTimeoutCheckSec' in output
    # import pdb; pdb.set_trace()
    assert output['MaxStorageVdsTimeoutCheckSec'] == ['30']
    assert output.get('MaxStorageVdsTimeoutCheckSec') == ['30']
    assert output.head('MaxStorageVdsTimeoutCheckSec') == '30'
    assert output.get_version('MaxStorageVdsTimeoutCheckSec') == ['general']

    # Keyword with mutiple values and versions
    assert output['HotUnplugCpuSupported'] == ['{"x86":"false","ppc":"false"}',
                                               '{"x86":"false","ppc":"false"}',
                                               '{"x86":"true","ppc":"true"}']
    assert output.last('HotUnplugCpuSupported') == '{"x86":"true","ppc":"true"}'  # noqa

    # Keyword with no value but version
    assert 'IPTablesConfigSiteCustom' in output
    assert output['IPTablesConfigSiteCustom'] == []
    assert output.get_version('IPTablesConfigSiteCustom') == ['general']

    # Keyword with no values but versions
    assert output['ClusterRequiredRngSourcesDefault'] == []
    assert output.get_version('ClusterRequiredRngSourcesDefault') == ['3.6',
                                                                      '4.0',
                                                                      '4.1']
    assert 'MigrationPolicies' in output
    assert output.get_version('MigrationPolicies') == ['3.6', '4.0', '4.1']

    # A keyword having multi-line output is not considered at-the-moment
    assert 'IPTablesConfig' not in output
    assert output['IPTablesConfig'] == []

    # Just another keyword
    assert 'UserSessionTimeOutInterval' in output
    assert output['UserSessionTimeOutInterval'] == ['30']
    assert output.get_version('UserSessionTimeOutInterval') == ['general']

    # A non existing keyword
    assert 'IDoNotExit' not in output
    assert output['IDoNotExit'] == []
    assert output.get('IDoNotExit') == []
    assert output.head('IDoNotExit') is None
    assert output.last('IDoNotExit') is None
    assert output.get_version('IDoNotExit') == []



from insights.parsers.pacemaker_log import PacemakerLog
from insights.tests import context_wrap

from datetime import datetime

PACEMAKER_LOG = """
Aug 21 12:58:32 [11661] example.redhat.com       crmd:     info: crm_timer_popped: 	PEngine Recheck Timer (I_PE_CALC) just popped (900000ms)
Aug 21 12:58:32 [11661] example.redhat.com       crmd:   notice: do_state_transition: 	State transition S_IDLE -> S_POLICY_ENGINE [ input=I_PE_CALC cause=C_TIMER_POPPED origin=crm_timer_popped ]
Aug 21 12:58:32 [11661] example.redhat.com       crmd:     info: do_state_transition: 	Progressed to state S_POLICY_ENGINE after C_TIMER_POPPED
Aug 21 12:58:32 [11656] example.redhat.com        cib:     info: cib_process_request: 	Completed cib_query operation for section 'all': OK (rc=0, origin=local/crmd/262, version=0.10.3)
Aug 21 12:58:32 [11660] example.redhat.com    pengine:   notice: unpack_config: 	On loss of CCM Quorum: Ignore
Aug 21 12:58:32 [11660] example.redhat.com    pengine:     info: determine_online_status: 	Node d-d9ckmw1 is online
Aug 21 12:58:32 [11660] example.redhat.com    pengine:     info: determine_online_status: 	Node d-gqynnw1 is online
Aug 21 12:58:32 [11660] example.redhat.com    pengine:   notice: stage6: 	Delaying fencing operations until there are resources to manage
Aug 21 12:58:32 [11660] example.redhat.com    pengine:   notice: process_pe_message: 	Calculated Transition 125: /var/lib/pacemaker/pengine/pe-input-14.bz2
Aug 21 12:58:33 [11661] example.redhat.com       crmd:     info: do_state_transition: 	State transition S_POLICY_ENGINE -> S_TRANSITION_ENGINE [ input=I_PE_SUCCESS
"""


def test_pacemaker_log():
    pacemaker = PacemakerLog(context_wrap(PACEMAKER_LOG))
    assert "Progressed to state S_POLICY_ENGINE after C_TIMER_POPPED" in pacemaker
    assert len(list(pacemaker.get_after(datetime(2017, 8, 21, 12, 58, 30)))) == 10



"""
AwxManage - commands ``awx-manage``
===================================

Parsers contains in this module are:

AnsibleTowerLicenseType - command ``/usr/bin/awx-manage check_license``

AnsibleTowerLicense - command ``/usr/bin/awx-manage check_license --data``

AwxManagePrintSettings - command ``/usr/bin/awx-manage print_settings``
"""

from insights import JSONParser, parser, CommandParser
from insights.parsers import SkipException, ParseException
from insights.specs import Specs


@parser(Specs.awx_manage_check_license)
class AnsibleTowerLicenseType(CommandParser, JSONParser):
    """
    Parses the output of command  ``/usr/bin/awx-manage check_license``

    Sample output of the command::

        enterprise

    Attributes:
        type (str): The license type, e.g. "enterprise"

    Examples:
        >>> type(awx_license)
        <class 'insights.parsers.awx_manage.AnsibleTowerLicenseType'>
        >>> awx_license.type == "enterprise"
        True
    """
    def parse_content(self, content):
        if not content:
            raise SkipException
        if len(content) != 1:
            raise ParseException("Invalid output: {0}".format(content))
        self.type = content[0].strip()


@parser(Specs.awx_manage_check_license_data)
class AnsibleTowerLicense(CommandParser, JSONParser):
    """
    Parses the output of command  ``/usr/bin/awx-manage check_license --data``

    Sample output of the command::

        {"instance_count": 100, "license_date": 1655092799, "license_type": "enterprise", "support_level": "Standard", "time_remaining": 29885220, "trial": false, "grace_period_remaining": 32477220, "compliant": true, "date_warning": false, "date_expired": false}

    Examples:
        >>> type(awx_manage_license)
        <class 'insights.parsers.awx_manage.AnsibleTowerLicense'>
        >>> awx_manage_license.data['license_type'] == "enterprise"
        True
        >>> awx_manage_license.data['time_remaining']
        29885220
    """
    pass


@parser(Specs.awx_manage_print_settings)
class AwxManagePrintSettings(CommandParser, JSONParser):
    """
    The AwxManagePrintSettings class parses the command ``awx-manage print_settings INSIGHTS_TRACKING_STATE SYSTEM_UUID INSTALL_UUID TOWER_URL_BASE AWX_CLEANUP_PATHS AWX_PROOT_BASE_PATH --format json``

    Sample command output::

        {
            "AWX_CLEANUP_PATHS": false,
            "AWX_PROOT_BASE_PATH": "/opt/tmp",
            "INSIGHTS_TRACKING_STATE": true,
            "INSTALL_UUID": "c0d38a6a-4449-4e13-a64b-00e0248ad229",
            "SYSTEM_UUID": "eecfd8dc-5028-46ef-9868-86f7d595da13",
            "TOWER_URL_BASE": "https://10.72.37.79",
            "LOG_AGGREGATOR_ENABLED": true,
            "LOG_AGGREGATOR_LEVEL": "DEBUG"

        }

    Examples:
        >>> type(settings)
        <class 'insights.parsers.awx_manage.AwxManagePrintSettings'>
        >>> settings['AWX_CLEANUP_PATHS']
        False
        >>> settings['SYSTEM_UUID'] == 'eecfd8dc-5028-46ef-9868-86f7d595da13'
        True
        >>> settings['LOG_AGGREGATOR_ENABLED']
        True
        >>> settings['LOG_AGGREGATOR_LEVEL'] == 'DEBUG'
        True
    """
    pass



"""
PackageProvidesJavaAll - Combiner for packages which provide java
=================================================================

.. warning::

    This module is deprecated, please use
    :mod:`insights.parsers.package_provides` instead.

Combiner for collecting all the java command and the corresponding package name
which is parsed by the PackageProvidesJava parser.

"""

from .. import LegacyItemAccess
from insights.core.plugins import combiner
from insights.parsers.package_provides_java import PackageProvidesJava
from insights.util import deprecated


@combiner(PackageProvidesJava)
class PackageProvidesJavaAll(LegacyItemAccess):
    """
    .. warning::

        This Combiner is deprecated, please use
        :class:`insights.parsers.package_provides.PackageProvidesCommand` Parser instead.

    Combiner for collecting all the java command and the corresponding package
    name which is parsed by the PackageProvidesJava parser.
    It works as a ``dict`` with the java command as the key and the
    corresponding package name as the value.

    Examples:
        >>> PACKAGE_COMMAND_MATCH_1 = '''/usr/lib/jvm/jre/bin/java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64'''
        >>> PACKAGE_COMMAND_MATCH_2 = '''/usr/lib/jvm/java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64/bin/java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64'''
        >>> pack1 = PackageProvidesJava(context_wrap(PACKAGE_COMMAND_MATCH_1))
        >>> pack2 = PackageProvidesJava(context_wrap(PACKAGE_COMMAND_MATCH_2))
        >>> shared = [{PackageProvidesJavaAll: [pack1, pack2]}]
        >>> packages = shared[PackageProvidesJavaAll]
        >>> packages.running_javas
        ['/usr/lib/jvm/jre/bin/java',
         '/usr/lib/jvm/java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64/bin/java']
        >>> packages.get_package("/usr/lib/jvm/jre/bin/java")
        'java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64'
        >>> packages.get("/usr/lib/jvm/jre/bin/java")
        'java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64'
        >>> packages["/usr/lib/jvm/jre/bin/java"]
        'java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64'

    """

    def __init__(self, package_provides_java):
        deprecated(
            PackageProvidesJavaAll,
            'Please use the :class:`insights.parsers.package_provides.PackageProvidesCommand` instead.'
        )
        self.data = {}
        for pkg in package_provides_java:
            self.data[pkg.command] = pkg.package
        super(PackageProvidesJavaAll, self).__init__()

    @property
    def running_javas(self):
        """
        Returns the list of java commands which are running on the system.
        """
        return self.data.keys()

    def get_package(self, java_command):
        """
        Returns the installed java package that provides the specified `java_command`.

        Parameters:
            java_command (str): The specified java command, e.g. found in ``ps`` command.

        Returns:
            (str): The package that provides the java command.
        """

        return self.data.get(java_command)
